"""
日付フィルター定義

日付プロパティに対するフィルター条件。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type DatePropertyFilter =
  | { equals: string }
  | { before: string }
  | { after: string }
  | { on_or_before: string }
  | { on_or_after: string }
  | { this_week: EmptyObject }
  | { past_week: EmptyObject }
  | { past_month: EmptyObject }
  | { past_year: EmptyObject }
  | { next_week: EmptyObject }
  | { next_month: EmptyObject }
  | { next_year: EmptyObject }
  | ExistencePropertyFilter
"""

from typing import TypedDict, Union

from .base_filters import ExistencePropertyFilter


# EmptyObject for relative date filters
class EmptyObject(TypedDict):
    """空オブジェクト（相対日付フィルター用）"""

    pass


# ============================================================================
# Date Property Filters
# ============================================================================


class DateFilterEquals(TypedDict):
    """日付が等しい

    Examples:
        ```python
        filter: DateFilterEquals = {"equals": "2025-01-01"}
        ```
    """

    equals: str


class DateFilterBefore(TypedDict):
    """日付より前"""

    before: str


class DateFilterAfter(TypedDict):
    """日付より後"""

    after: str


class DateFilterOnOrBefore(TypedDict):
    """日付以前"""

    on_or_before: str


class DateFilterOnOrAfter(TypedDict):
    """日付以降"""

    on_or_after: str


class DateFilterThisWeek(TypedDict):
    """今週

    Examples:
        ```python
        filter: DateFilterThisWeek = {"this_week": {}}
        ```
    """

    this_week: EmptyObject


class DateFilterPastWeek(TypedDict):
    """先週"""

    past_week: EmptyObject


class DateFilterPastMonth(TypedDict):
    """先月"""

    past_month: EmptyObject


class DateFilterPastYear(TypedDict):
    """昨年"""

    past_year: EmptyObject


class DateFilterNextWeek(TypedDict):
    """来週"""

    next_week: EmptyObject


class DateFilterNextMonth(TypedDict):
    """来月"""

    next_month: EmptyObject


class DateFilterNextYear(TypedDict):
    """来年"""

    next_year: EmptyObject


# Matches TypeScript DatePropertyFilter
DatePropertyFilter = Union[
    DateFilterEquals,
    DateFilterBefore,
    DateFilterAfter,
    DateFilterOnOrBefore,
    DateFilterOnOrAfter,
    DateFilterThisWeek,
    DateFilterPastWeek,
    DateFilterPastMonth,
    DateFilterPastYear,
    DateFilterNextWeek,
    DateFilterNextMonth,
    DateFilterNextYear,
    ExistencePropertyFilter,
]



"""
特殊ブロックの定義

SyncedBlock, ChildPage, ChildDatabase, Equation, Code, Callout
"""

from __future__ import annotations

from enum import Enum
from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .base import ApiColor, BaseBlockObject, BlockType
from ..models.rich_text_item import RichTextItem
from ..models.icon import NotionIcon


class CodeLanguage(str, Enum):
    """コードブロックの言語"""

    ABAP = "abap"
    ABC = "abc"
    AGDA = "agda"
    ARDUINO = "arduino"
    ASCII_ART = "ascii art"
    ASSEMBLY = "assembly"
    BASH = "bash"
    BASIC = "basic"
    BNF = "bnf"
    C = "c"
    C_SHARP = "c#"
    C_PLUS_PLUS = "c++"
    CLOJURE = "clojure"
    COFFEESCRIPT = "coffeescript"
    COQ = "coq"
    CSS = "css"
    DART = "dart"
    DHALL = "dhall"
    DIFF = "diff"
    DOCKER = "docker"
    EBNF = "ebnf"
    ELIXIR = "elixir"
    ELM = "elm"
    ERLANG = "erlang"
    F_SHARP = "f#"
    FLOW = "flow"
    FORTRAN = "fortran"
    GHERKIN = "gherkin"
    GLSL = "glsl"
    GO = "go"
    GRAPHQL = "graphql"
    GROOVY = "groovy"
    HASKELL = "haskell"
    HCL = "hcl"
    HTML = "html"
    IDRIS = "idris"
    JAVA = "java"
    JAVASCRIPT = "javascript"
    JSON = "json"
    JULIA = "julia"
    KOTLIN = "kotlin"
    LATEX = "latex"
    LESS = "less"
    LISP = "lisp"
    LIVESCRIPT = "livescript"
    LLVM_IR = "llvm ir"
    LUA = "lua"
    MAKEFILE = "makefile"
    MARKDOWN = "markdown"
    MARKUP = "markup"
    MATLAB = "matlab"
    MATHEMATICA = "mathematica"
    MERMAID = "mermaid"
    NIX = "nix"
    NOTION_FORMULA = "notion formula"
    OBJECTIVE_C = "objective-c"
    OCAML = "ocaml"
    PASCAL = "pascal"
    PERL = "perl"
    PHP = "php"
    PLAIN_TEXT = "plain text"
    POWERSHELL = "powershell"
    PROLOG = "prolog"
    PROTOBUF = "protobuf"
    PURESCRIPT = "purescript"
    PYTHON = "python"
    R = "r"
    RACKET = "racket"
    REASON = "reason"
    RUBY = "ruby"
    RUST = "rust"
    SASS = "sass"
    SCALA = "scala"
    SCHEME = "scheme"
    SCSS = "scss"
    SHELL = "shell"
    SMALLTALK = "smalltalk"
    SOLIDITY = "solidity"
    SQL = "sql"
    SWIFT = "swift"
    TOML = "toml"
    TYPESCRIPT = "typescript"
    VB_NET = "vb.net"
    VERILOG = "verilog"
    VHDL = "vhdl"
    VISUAL_BASIC = "visual basic"
    WEBASSEMBLY = "webassembly"
    XML = "xml"
    YAML = "yaml"
    JAVA_C_CPP_CSHARP = "java/c/c++/c#"


# ============================================
# Content Models
# ============================================


class SyncedBlockContent(BaseModel):
    """同期ブロックコンテンツ"""

    synced_from: SyncedFromBlock | None = Field(None, description="同期元ブロック")


class SyncedFromBlock(BaseModel):
    """同期元ブロック情報"""

    type: Literal["block_id"] = Field("block_id", description="タイプ")
    block_id: StrictStr = Field(..., description="ブロックID")


class TitleObject(BaseModel):
    """タイトルオブジェクト"""

    title: StrictStr = Field(..., description="タイトル")


class ExpressionObject(BaseModel):
    """数式オブジェクト"""

    expression: StrictStr = Field(..., description="数式")


class CodeContent(BaseModel):
    """コードコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    language: CodeLanguage = Field(..., description="プログラミング言語")


class CalloutContent(BaseModel):
    """コールアウトコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    color: ApiColor = Field(..., description="カラー設定")
    icon: NotionIcon | None = Field(None, description="アイコン")


# ============================================
# Special Blocks
# ============================================


class SyncedBlockBlock(BaseBlockObject):
    """同期ブロック"""

    type: Literal[BlockType.SYNCED_BLOCK] = Field(
        BlockType.SYNCED_BLOCK, description="ブロックタイプ"
    )
    synced_block: SyncedBlockContent = Field(..., description="同期ブロックコンテンツ")


class ChildPageBlock(BaseBlockObject):
    """子ページブロック"""

    type: Literal[BlockType.CHILD_PAGE] = Field(
        BlockType.CHILD_PAGE, description="ブロックタイプ"
    )
    child_page: TitleObject = Field(..., description="子ページ情報")


class ChildDatabaseBlock(BaseBlockObject):
    """子データベースブロック"""

    type: Literal[BlockType.CHILD_DATABASE] = Field(
        BlockType.CHILD_DATABASE, description="ブロックタイプ"
    )
    child_database: TitleObject = Field(..., description="子データベース情報")


class EquationBlock(BaseBlockObject):
    """数式ブロック"""

    type: Literal[BlockType.EQUATION] = Field(
        BlockType.EQUATION, description="ブロックタイプ"
    )
    equation: ExpressionObject = Field(..., description="数式")


class CodeBlock(BaseBlockObject):
    """コードブロック"""

    type: Literal[BlockType.CODE] = Field(BlockType.CODE, description="ブロックタイプ")
    code: CodeContent = Field(..., description="コードコンテンツ")


class CalloutBlock(BaseBlockObject):
    """コールアウトブロック"""

    type: Literal[BlockType.CALLOUT] = Field(
        BlockType.CALLOUT, description="ブロックタイプ"
    )
    callout: CalloutContent = Field(..., description="コールアウトコンテンツ")



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr

from .primitives import EmptyObject


class Person(BaseModel):
    """personタイプの詳細情報"""

    email: StrictStr | None = Field(None, description="メールアドレス")


class PartialUser(BaseModel):
    """Notionの部分ユーザー情報"""

    object: Literal["user"] = Field("user", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ユーザーID")


class BaseFullUser(PartialUser):
    """共通のユーザー情報"""

    name: StrictStr | None = Field(None, description="ユーザー名")
    avatar_url: StrictStr | None = Field(None, description="アバターURL")


class PersonUser(BaseFullUser):
    """personタイプのユーザー"""

    type: Literal["person"] = Field("person", description="ユーザータイプ")
    person: Person | None = Field(None, description="personタイプの詳細")


class BotOwnerUser(BaseModel):
    """botユーザーのオーナーがユーザーの場合の情報"""

    type: Literal["user"] = Field("user", description="オーナータイプ")
    user: PartialUser | PersonUser = Field(..., description="オーナーユーザー情報")


class BotOwnerWorkspace(BaseModel):
    """botユーザーのオーナーがワークスペースの場合の情報"""

    type: Literal["workspace"] = Field(
        "workspace", description="オーナータイプ"
    )
    workspace: StrictBool = Field(True, description="ワークスペースフラグ")


class BotWorkspaceLimits(BaseModel):
    """ワークスペースの制限情報"""

    max_file_upload_size_in_bytes: StrictInt = Field(
        ..., description="最大アップロードサイズ（バイト）"
    )


class BotInfo(BaseModel):
    """botタイプユーザーの詳細情報"""

    owner: BotOwnerUser | BotOwnerWorkspace = Field(
        ..., description="Botのオーナー情報"
    )
    workspace_name: StrictStr | None = Field(
        None, description="Botの所属ワークスペース名"
    )
    workspace_limits: BotWorkspaceLimits = Field(
        ..., description="ワークスペースの制限"
    )


class BotUser(BaseFullUser):
    """botタイプのユーザー"""

    type: Literal["bot"] = Field("bot", description="ユーザータイプ")
    bot: EmptyObject | BotInfo = Field(
        default_factory=EmptyObject, description="Bot固有の情報"
    )


User = PersonUser | BotUser
"""NotionのユーザーのUnion型"""


class Group(BaseModel):
    """Notionのグループ情報"""

    object: Literal["group"] = Field("group", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="グループID")
    name: StrictStr | None = Field(None, description="グループ名")



from __future__ import annotations

from typing import Literal, Self

from pydantic import BaseModel, Field, StrictStr, model_validator

from .base_configs import BasePropertyConfig
from ...properties.base_properties._base_property import NotionPropertyType


class RollupSettings(BaseModel):
    """rollupプロパティの設定"""

    function: StrictStr = Field(..., description="集計関数")
    relation_property_name: StrictStr | None = Field(
        None, description="リレーションプロパティ名"
    )
    relation_property_id: StrictStr | None = Field(
        None, description="リレーションプロパティID"
    )
    rollup_property_name: StrictStr | None = Field(
        None, description="ロールアップ対象プロパティ名"
    )
    rollup_property_id: StrictStr | None = Field(
        None, description="ロールアップ対象プロパティID"
    )

    @model_validator(mode="after")
    def validate_references(self) -> Self:
        """relation_property_idまたはrelation_property_nameのいずれかが必須であることを検証"""
        if not self.relation_property_id and not self.relation_property_name:
            raise ValueError(
                "RollupSettings requires either relation_property_id or relation_property_name"
            )
        if not self.rollup_property_id and not self.rollup_property_name:
            raise ValueError(
                "RollupSettings requires either rollup_property_id or rollup_property_name"
            )
        return self


class RollupPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.ROLLUP]]):
    """Notionのrollupプロパティ設定"""

    type: Literal[NotionPropertyType.ROLLUP] = Field(
        NotionPropertyType.ROLLUP, description="プロパティタイプ"
    )
    rollup: RollupSettings



from typing import Literal
from pydantic import Field, StrictFloat, StrictInt

from ._base_property import BaseProperty, NotionPropertyType


class NumberProperty(BaseProperty[Literal[NotionPropertyType.NUMBER]]):
    """Notionのnumberプロパティ"""

    type: Literal[NotionPropertyType.NUMBER] = Field(
        NotionPropertyType.NUMBER, description="プロパティタイプ"
    )

    number: StrictInt | StrictFloat | None = Field(
        None, description="数値（設定されていない場合はnull）"
    )

    def get_display_value(self) -> int | float | None:
        """数値を取得

        Returns:
            StrictInt | StrictFloat | None: 数値（未設定の場合はnull）
        """
        return self.number



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from ._base_config import BasePropertyConfig
from ....properties.base_properties._base_property import NotionPropertyType


class FormulaExpression(BaseModel):
    """formulaプロパティの式設定"""

    expression: StrictStr | None = Field(None, description="Notionの式")


class FormulaPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.FORMULA]]):
    """Notionのformulaプロパティ設定"""

    type: Literal[NotionPropertyType.FORMULA] = Field(
        NotionPropertyType.FORMULA, description="プロパティタイプ"
    )
    formula: FormulaExpression = Field(..., description="formula設定")



"""
Notion API フィルター型定義

データベースクエリ時のフィルタ条件を型安全に定義する。
TypedDict を使用して公式 Notion API 仕様に準拠。

主要なエクスポート:
- PropertyFilter: プロパティフィルター統合型
- TimestampFilter: タイムスタンプフィルター
- create_and_filter/create_or_filter: 複合フィルターヘルパー
- 各種具体的なフィルタータイプ

使用例:
    ```python
    from notion_py_client.filters import create_and_filter

    # TypedDict形式で型安全にフィルター作成
    filter = create_and_filter(
        {"property": "Status", "status": {"equals": "Active"}},
        {"property": "Amount", "number": {"greater_than": 10000}},
    )
    ```
"""

# 基本フィルター
from .base_filters import (
    ExistenceFilterEmpty,
    ExistenceFilterNotEmpty,
    ExistencePropertyFilter,
)

# 値フィルター
from .value_filters import (
    CheckboxFilterEquals,
    CheckboxFilterDoesNotEqual,
    CheckboxPropertyFilter,
    MultiSelectFilterContains,
    MultiSelectFilterDoesNotContain,
    MultiSelectPropertyFilter,
    NumberFilterDoesNotEqual,
    NumberFilterEquals,
    NumberFilterGreaterThan,
    NumberFilterGreaterThanOrEqualTo,
    NumberFilterLessThan,
    NumberFilterLessThanOrEqualTo,
    NumberPropertyFilter,
    SelectFilterDoesNotEqual,
    SelectFilterEquals,
    SelectPropertyFilter,
    StatusFilterDoesNotEqual,
    StatusFilterEquals,
    StatusPropertyFilter,
    TextFilterContains,
    TextFilterDoesNotContain,
    TextFilterDoesNotEqual,
    TextFilterEndsWith,
    TextFilterEquals,
    TextFilterStartsWith,
    TextPropertyFilter,
)

# 日付フィルター
from .date_filters import (
    DateFilterAfter,
    DateFilterBefore,
    DateFilterEquals,
    DateFilterNextMonth,
    DateFilterNextWeek,
    DateFilterNextYear,
    DateFilterOnOrAfter,
    DateFilterOnOrBefore,
    DateFilterPastMonth,
    DateFilterPastWeek,
    DateFilterPastYear,
    DateFilterThisWeek,
    DatePropertyFilter,
)

# Relation/Peopleフィルター
from .relation_filters import (
    PeopleFilterContains,
    PeopleFilterDoesNotContain,
    PeoplePropertyFilter,
    RelationFilterContains,
    RelationFilterDoesNotContain,
    RelationPropertyFilter,
)

# 高度なフィルター
from .advanced_filters import (
    FormulaFilterCheckbox,
    FormulaFilterDate,
    FormulaFilterNumber,
    FormulaFilterString,
    FormulaPropertyFilter,
    RollupFilterAny,
    RollupFilterDate,
    RollupFilterEvery,
    RollupFilterNone,
    RollupFilterNumber,
    RollupPropertyFilter,
    RollupSubfilterCheckbox,
    RollupSubfilterDate,
    RollupSubfilterFiles,
    RollupSubfilterMultiSelect,
    RollupSubfilterNumber,
    RollupSubfilterPeople,
    RollupSubfilterPropertyFilter,
    RollupSubfilterRelation,
    RollupSubfilterRichText,
    RollupSubfilterSelect,
    RollupSubfilterStatus,
    VerificationPropertyStatusFilter,
)

# プロパティフィルター
from .property_filters import (
    PropertyFilter,
    PropertyFilterCheckbox,
    PropertyFilterCreatedBy,
    PropertyFilterCreatedTime,
    PropertyFilterDate,
    PropertyFilterEmail,
    PropertyFilterFiles,
    PropertyFilterFormula,
    PropertyFilterLastEditedBy,
    PropertyFilterLastEditedTime,
    PropertyFilterMultiSelect,
    PropertyFilterNumber,
    PropertyFilterPeople,
    PropertyFilterPhoneNumber,
    PropertyFilterRelation,
    PropertyFilterRichText,
    PropertyFilterRollup,
    PropertyFilterSelect,
    PropertyFilterStatus,
    PropertyFilterTitle,
    PropertyFilterUniqueId,
    PropertyFilterUrl,
    PropertyFilterVerification,
)

# タイムスタンプフィルター
from .timestamp_filters import (
    TimestampCreatedTimeFilter,
    TimestampFilter,
    TimestampLastEditedTimeFilter,
)

# 複合フィルター
from .compound_filters import (
    AndFilterDict,
    FilterCondition,
    OrFilterDict,
    create_and_filter,
    create_or_filter,
)

__all__ = [
    # 基本フィルター
    "ExistenceFilterEmpty",
    "ExistenceFilterNotEmpty",
    "ExistencePropertyFilter",
    # テキストフィルター
    "TextFilterEquals",
    "TextFilterDoesNotEqual",
    "TextFilterContains",
    "TextFilterDoesNotContain",
    "TextFilterStartsWith",
    "TextFilterEndsWith",
    "TextPropertyFilter",
    # 数値フィルター
    "NumberFilterEquals",
    "NumberFilterDoesNotEqual",
    "NumberFilterGreaterThan",
    "NumberFilterLessThan",
    "NumberFilterGreaterThanOrEqualTo",
    "NumberFilterLessThanOrEqualTo",
    "NumberPropertyFilter",
    # チェックボックスフィルター
    "CheckboxFilterEquals",
    "CheckboxFilterDoesNotEqual",
    "CheckboxPropertyFilter",
    # セレクトフィルター
    "SelectFilterEquals",
    "SelectFilterDoesNotEqual",
    "SelectPropertyFilter",
    # マルチセレクトフィルター
    "MultiSelectFilterContains",
    "MultiSelectFilterDoesNotContain",
    "MultiSelectPropertyFilter",
    # ステータスフィルター
    "StatusFilterEquals",
    "StatusFilterDoesNotEqual",
    "StatusPropertyFilter",
    # 日付フィルター
    "DateFilterEquals",
    "DateFilterBefore",
    "DateFilterAfter",
    "DateFilterOnOrBefore",
    "DateFilterOnOrAfter",
    "DateFilterThisWeek",
    "DateFilterPastWeek",
    "DateFilterPastMonth",
    "DateFilterPastYear",
    "DateFilterNextWeek",
    "DateFilterNextMonth",
    "DateFilterNextYear",
    "DatePropertyFilter",
    # Peopleフィルター
    "PeopleFilterContains",
    "PeopleFilterDoesNotContain",
    "PeoplePropertyFilter",
    # Relationフィルター
    "RelationFilterContains",
    "RelationFilterDoesNotContain",
    "RelationPropertyFilter",
    # Formulaフィルター
    "FormulaFilterString",
    "FormulaFilterCheckbox",
    "FormulaFilterNumber",
    "FormulaFilterDate",
    "FormulaPropertyFilter",
    # Rollupフィルター
    "RollupFilterAny",
    "RollupFilterNone",
    "RollupFilterEvery",
    "RollupFilterDate",
    "RollupFilterNumber",
    "RollupPropertyFilter",
    "RollupSubfilterRichText",
    "RollupSubfilterNumber",
    "RollupSubfilterCheckbox",
    "RollupSubfilterSelect",
    "RollupSubfilterMultiSelect",
    "RollupSubfilterRelation",
    "RollupSubfilterDate",
    "RollupSubfilterPeople",
    "RollupSubfilterFiles",
    "RollupSubfilterStatus",
    "RollupSubfilterPropertyFilter",
    # Verificationフィルター
    "VerificationPropertyStatusFilter",
    # プロパティフィルター
    "PropertyFilterTitle",
    "PropertyFilterRichText",
    "PropertyFilterNumber",
    "PropertyFilterCheckbox",
    "PropertyFilterSelect",
    "PropertyFilterMultiSelect",
    "PropertyFilterStatus",
    "PropertyFilterDate",
    "PropertyFilterPeople",
    "PropertyFilterFiles",
    "PropertyFilterUrl",
    "PropertyFilterEmail",
    "PropertyFilterPhoneNumber",
    "PropertyFilterRelation",
    "PropertyFilterCreatedBy",
    "PropertyFilterCreatedTime",
    "PropertyFilterLastEditedBy",
    "PropertyFilterLastEditedTime",
    "PropertyFilterFormula",
    "PropertyFilterUniqueId",
    "PropertyFilterRollup",
    "PropertyFilterVerification",
    "PropertyFilter",
    # タイムスタンプフィルター
    "TimestampCreatedTimeFilter",
    "TimestampLastEditedTimeFilter",
    "TimestampFilter",
    # 複合フィルター
    "AndFilterDict",
    "OrFilterDict",
    "FilterCondition",
    "create_and_filter",
    "create_or_filter",
]



from typing import Literal

from pydantic import Field

from ...models import Verification
from ._base_property import BaseProperty, NotionPropertyType


class VerificationProperty(BaseProperty[Literal[NotionPropertyType.VERIFICATION]]):
    """Notionのverificationプロパティ"""

    type: Literal[NotionPropertyType.VERIFICATION] = Field(
        NotionPropertyType.VERIFICATION, description="プロパティタイプ"
    )

    verification: Verification | None = Field(None, description="検証状態情報")

    def get_display_value(self) -> str | None:
        """検証状態情報を取得

        Returns:
            str | None: 検証状態情報の文字列表現。未設定の場合はNone
        """
        if self.verification is None:
            return None
        return self.verification.state



from typing import Literal
from pydantic import BaseModel, Field, StrictBool, StrictStr

from .base_properties._base_property import NotionPropertyType

from .base_properties import BaseProperty


class RelationItem(BaseModel):
    """Notionのrelation項目"""

    id: StrictStr = Field(..., description="関連項目ID")


class RelationProperty(BaseProperty[Literal[NotionPropertyType.RELATION]]):
    """Notionのrelationプロパティ"""

    type: Literal[NotionPropertyType.RELATION] = Field(
        NotionPropertyType.RELATION, description="プロパティタイプ"
    )

    relation: list[RelationItem] = Field(
        default_factory=list, description="関連項目配列"
    )
    has_more: StrictBool = Field(False, description="さらに関連項目があるかどうか")

    def get_display_value(self) -> str | None:
        """関連項目のIDリストをカンマ区切りで取得

        Returns:
            StrictStr | None: 関連項目のIDをカンマ区切りで連結した文字列。関連項目がない場合はNone
        """
        if len(self.relation) == 0:
            return None
        return ", ".join(item.id for item in self.relation)



from pydantic import BaseModel, Field, StrictStr


class CustomEmoji(BaseModel):
    """Notionのcustom_emojiメンション"""

    id: StrictStr = Field(..., description="カスタム絵文字ID")
    name: StrictStr | None = Field(None, description="カスタム絵文字名")
    url: StrictStr | None = Field(None, description="カスタム絵文字URL")



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class EmailPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.EMAIL]]):
    """Notionのemailプロパティ設定"""

    type: Literal[NotionPropertyType.EMAIL] = Field(
        NotionPropertyType.EMAIL, description="プロパティタイプ"
    )
    email: EmptyObject = Field(
        default_factory=EmptyObject, description="email設定"
    )



from __future__ import annotations

import re
from typing import (
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    Mapping,
    Sequence,
    TypeVar,
    Protocol,
    TypeGuard,
    Literal,
)

try:
    from pydantic import BaseModel  # type: ignore
except Exception:  # pragma: no cover - pydantic should exist, but make robust
    BaseModel = object  # type: ignore


# ========= Pagination helpers =========
TItem = TypeVar("TItem")


class _PaginatedResponseLike(Protocol[TItem]):
    """Structural protocol for Notion list responses.

    Works with both dict responses and Pydantic models that expose
    `results`, `next_cursor`, and `has_more` attributes.
    """

    results: Sequence[TItem]
    next_cursor: str | None
    has_more: bool


def _get_attr(obj: Any, key: str, default: Any = None) -> Any:
    """Get attribute or dict item from `obj` with a default."""
    if isinstance(obj, Mapping):
        return obj.get(key, default)
    return getattr(obj, key, default)


async def iterate_paginated_api(
    list_fn: Callable[..., Awaitable[_PaginatedResponseLike[TItem]]],
    first_page_args: Mapping[str, Any] | None = None,
) -> AsyncIterator[TItem]:
    """Iterate over items from any Notion paginated API.

    Mirrors the official TypeScript helper `iteratePaginatedAPI`.

    Example:
        ```python
        async for block in iterate_paginated_api(
            notion.blocks.children.list, {"block_id": parent_block_id}
        ):
            ...
        ```

    Args:
        list_fn: Bound method to a Notion client that returns a paginated list
            object with `results`, `next_cursor`, and `has_more`.
        first_page_args: Arguments to pass on the first and subsequent calls.
            If provided, `start_cursor` will be injected/overwritten as needed.
    """

    args = dict(first_page_args or {})
    next_cursor: str | None = args.get("start_cursor")
    while True:
        # Always pass start_cursor; methods accept None for the first page.
        page_args = {**args, "start_cursor": next_cursor}
        response = await list_fn(**page_args)
        results: Sequence[TItem] = _get_attr(response, "results", [])
        for item in results:
            yield item
        next_cursor = _get_attr(response, "next_cursor")
        if not next_cursor:
            break


async def collect_paginated_api(
    list_fn: Callable[..., Awaitable[_PaginatedResponseLike[TItem]]],
    first_page_args: Mapping[str, Any] | None = None,
) -> list[TItem]:
    """Collect all results from a paginated Notion API into a list.

    Example:
        ```python
        blocks = await collect_paginated_api(
            notion.blocks.children.list, {"block_id": parent_id}
        )
        ```
    """

    out: list[TItem] = []
    async for item in iterate_paginated_api(list_fn, first_page_args):
        out.append(item)
    return out


# ========= Type predicate helpers (TypeScript-like type guards) =========
# Import concrete models for precise TypeGuard targets
from .responses.page import NotionPage, PartialPage
from .responses.datasource import DataSource, PartialDataSource
from .responses.database import NotionDatabase, PartialDatabase
from .blocks.base import BaseBlockObject, PartialBlock
from .models.rich_text_item import RichTextItem
from .models.primitives import Text, Mention, Equation


def _get_object_type(obj: Any) -> str | None:
    return _get_attr(obj, "object")


def is_full_block(
    response: BaseBlockObject | PartialBlock | Mapping[str, Any] | Any,
) -> TypeGuard[BaseBlockObject]:
    """Type guard: block object with full fields (`type` present)."""
    return (
        _get_object_type(response) == "block"
        and _get_attr(response, "type") is not None
    )


def is_full_page(
    response: NotionPage | PartialPage | Mapping[str, Any] | Any,
) -> TypeGuard[NotionPage]:
    """Type guard: page object with full fields (`url` present)."""
    return (
        _get_object_type(response) == "page" and _get_attr(response, "url") is not None
    )


def is_full_data_source(
    response: DataSource | PartialDataSource | Mapping[str, Any] | Any,
) -> TypeGuard[DataSource]:
    """Type guard: data source (full schema object)."""
    return (
        _get_object_type(response) == "data_source"
        and _get_attr(response, "url") is not None
    )


def is_full_database(
    response: NotionDatabase | PartialDatabase | Mapping[str, Any] | Any,
) -> TypeGuard[NotionDatabase]:
    """Type guard: database (full metadata object)."""
    return _get_object_type(response) == "database" and hasattr(response, "title")


def is_full_page_or_data_source(
    response: (
        NotionPage
        | PartialPage
        | DataSource
        | PartialDataSource
        | Mapping[str, Any]
        | Any
    ),
) -> TypeGuard[NotionPage | DataSource]:
    """Type guard: full Page or full DataSource."""
    obj = _get_object_type(response)
    if obj == "data_source":
        return is_full_data_source(response)
    return is_full_page(response)


from .models.user import PartialUser, User  # type: ignore


def is_full_user(
    response: PartialUser | User | Mapping[str, Any] | Any,
) -> TypeGuard[User]:
    """Type guard: full User object (has `type`)."""
    return _get_attr(response, "type") in ("person", "bot")


def is_full_comment(response: Mapping[str, Any] | Any) -> TypeGuard[Mapping[str, Any]]:
    """Best-effort type guard for comment dicts (created_by present)."""
    return isinstance(response, Mapping) and "created_by" in response


class TextRichTextItem(Protocol):
    type: Literal["text"]
    text: Text
    mention: None
    equation: None


class EquationRichTextItem(Protocol):
    type: Literal["equation"]
    equation: Equation
    text: None
    mention: None


class MentionRichTextItem(Protocol):
    type: Literal["mention"]
    mention: Mention
    text: None
    equation: None


def is_text_rich_text_item_response(
    rich_text: RichTextItem | Mapping[str, Any] | Any,
) -> TypeGuard[TextRichTextItem]:
    """Type guard: rich text item is `text` variant (ensures `.text` not None)."""
    return _get_attr(rich_text, "type") == "text"


def is_equation_rich_text_item_response(
    rich_text: RichTextItem | Mapping[str, Any] | Any,
) -> TypeGuard[EquationRichTextItem]:
    """Type guard: rich text item is `equation` variant (ensures `.equation` not None)."""
    return _get_attr(rich_text, "type") == "equation"


def is_mention_rich_text_item_response(
    rich_text: RichTextItem | Mapping[str, Any] | Any,
) -> TypeGuard[MentionRichTextItem]:
    """Type guard: rich text item is `mention` variant (ensures `.mention` not None)."""
    return _get_attr(rich_text, "type") == "mention"


# ========= Notion ID extractors =========

_UUID_HYPHEN_RE = re.compile(
    r"^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$",
    re.IGNORECASE,
)
_UUID_COMPACT_RE = re.compile(r"^[0-9a-f]{32}$", re.IGNORECASE)


def _format_uuid(compact_id: str) -> str:
    clean = compact_id.lower()
    return (
        f"{clean[0:8]}-"
        f"{clean[8:12]}-"
        f"{clean[12:16]}-"
        f"{clean[16:20]}-"
        f"{clean[20:32]}"
    )


def extract_notion_id(url_or_id: str | None) -> str | None:
    """Extract a Notion UUID from a URL or return the input if already a UUID.

    - Accepts hyphenated UUIDs and 32-char hex (no hyphens).
    - Prioritizes IDs found in the URL path over query parameters to avoid
      accidentally extracting view IDs from `v=` queries.

    Returns the UUID in standard hyphenated format or None if not found.
    """
    if not url_or_id or not isinstance(url_or_id, str):
        return None

    s = url_or_id.strip()
    if _UUID_HYPHEN_RE.fullmatch(s):
        return s.lower()
    if _UUID_COMPACT_RE.fullmatch(s):
        return _format_uuid(s)

    # Prefer path segment IDs like `/...-<32hex>`
    m = re.search(r"/[^/?#]*-([0-9a-f]{32})(?:[/?#]|$)", s, flags=re.IGNORECASE)
    if m and m.group(1):
        return _format_uuid(m.group(1))

    # Then query parameters like ?p=<32hex> or ?page_id=<32hex> or ?database_id=<32hex>
    m = re.search(
        r"[?&](?:p|page_id|database_id)=([0-9a-f]{32})", s, flags=re.IGNORECASE
    )
    if m and m.group(1):
        return _format_uuid(m.group(1))

    # Fallback: any 32-hex sequence
    m = re.search(r"([0-9a-f]{32})", s, flags=re.IGNORECASE)
    if m and m.group(1):
        return _format_uuid(m.group(1))

    return None


def extract_database_id(database_url: str | None) -> str | None:
    """Extract a database ID from a Notion database URL or return None."""
    return extract_notion_id(database_url)


def extract_page_id(page_url: str | None) -> str | None:
    """Extract a page ID from a Notion page URL or return None."""
    return extract_notion_id(page_url)


def extract_block_id(url_with_block: str | None) -> str | None:
    """Extract a block ID from a URL fragment like `#block-<id>` or `#<id>`."""
    if not url_with_block or not isinstance(url_with_block, str):
        return None
    m = re.search(r"#(?:block-)?([0-9a-f]{32})", url_with_block, flags=re.IGNORECASE)
    if m and m.group(1):
        return _format_uuid(m.group(1))
    return None



"""
Notionリストレスポンス型の定義

TypeScript定義: ListUsersResponse, QueryDatabaseResponse, ListBlockChildrenResponse等
"""

from typing import Generic, Literal, TypeVar

from pydantic import BaseModel, Field, StrictBool, StrictStr

from .page import NotionPage, PartialPage
from .database import NotionDatabase, PartialDatabase
from .datasource import DataSource, PartialDataSource
from .file_upload import FileUploadObject
from ..models.user import PartialUser

T = TypeVar("T")


class ListResponse(BaseModel, Generic[T]):
    """汎用リストレスポンス.

    Notionの全てのリストAPIで共通のページネーション構造。

    Attributes:
        object: 常に "list"
        results: 結果の配列
        next_cursor: 次のページのカーソル（なければNone）
        has_more: さらにページがあるかどうか

    Examples:
        >>> response = await client.databases.query({"database_id": "abc"})
        >>> print(f"Has more: {response.has_more}")
        >>> for page in response.results:
        ...     print(page.id)
    """

    object: Literal["list"] = Field(..., description="オブジェクトタイプ")
    results: list[T] = Field(..., description="結果の配列")
    next_cursor: StrictStr | None = Field(None, description="次のページのカーソル")
    has_more: StrictBool = Field(..., description="さらにページがあるか")
    type: StrictStr | None = Field(
        None, description="結果の型ヒント（例: 'page_or_database'）"
    )


# 具体的なリストレスポンス型
class QueryDatabaseResponse(ListResponse[NotionPage]):
    """databases.query() のレスポンス型.

    備考: リスト型の共通フィールド `type` は親クラスで定義済み。
    サブクラスではオーバーライドしない（LSP/型チェッカー警告を避ける）。
    """


class QueryDataSourceResponse(
    ListResponse[NotionPage | PartialPage | DataSource | PartialDataSource]
):
    """dataSources.query() のレスポンス型.

    備考: `type` は親クラス側の型（StrictStr | None）を使用。
    """


class ListUsersResponse(ListResponse[PartialUser]):
    """users.list() のレスポンス型.

    備考: `type` は親クラスに準拠。
    """


class SearchResponse(
    ListResponse[NotionPage | PartialPage | NotionDatabase | PartialDatabase]
):
    """search() のレスポンス型.

    備考: `type` は親クラスに準拠。
    """


class ListFileUploadsResponse(ListResponse[FileUploadObject]):
    """file_uploads.list() のレスポンス型.

    備考: `type` は親クラスに準拠（省略される場合あり）。
    """



from typing import Literal

from pydantic import BaseModel, Field


class TemplateMentionDate(BaseModel):
    """today/now を参照するテンプレートメンション"""

    type: Literal["template_mention_date"] = Field(
        "template_mention_date", description="テンプレートメンションタイプ"
    )
    template_mention_date: Literal["today", "now"] = Field(
        ..., description="参照される日付"
    )


class TemplateMentionUser(BaseModel):
    """me を参照するテンプレートメンション"""

    type: Literal["template_mention_user"] = Field(
        "template_mention_user", description="テンプレートメンションタイプ"
    )
    template_mention_user: Literal["me"] = Field(
        ..., description="参照されるユーザー"
    )


TemplateMention = TemplateMentionDate | TemplateMentionUser



from typing import Literal

from pydantic import Field, StrictStr

from ._base_property import BaseProperty, NotionPropertyType


class PhoneNumberProperty(BaseProperty[Literal[NotionPropertyType.PHONE_NUMBER]]):
    """Notionのphone_numberプロパティ"""

    type: Literal[NotionPropertyType.PHONE_NUMBER] = Field(
        NotionPropertyType.PHONE_NUMBER, description="プロパティタイプ"
    )

    phone_number: StrictStr | None = Field(
        None, description="電話番号（未設定の場合はnull）"
    )

    def get_display_value(self) -> str | None:
        """電話番号を取得

        Returns:
            StrictStr | None: 電話番号（未設定の場合はnull）
        """
        return self.phone_number



from pydantic import BaseModel, Field, StrictStr


class Link(BaseModel):
    """RichText内のリンク情報"""

    url: StrictStr = Field(..., description="リンクURL")


class Text(BaseModel):
    """NotionのRichTextのテキスト情報"""

    content: StrictStr = Field(..., description="テキスト内容")
    link: Link | None = Field(None, description="リンク情報（ない場合はnull）")



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class LocationPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.LOCATION]]
):
    """Notionのlocationプロパティ設定

    TypeScript: LocationPropertyConfigurationRequest
    フィールドは空オブジェクトで構成されます。
    """

    type: Literal[NotionPropertyType.LOCATION] = Field(
        NotionPropertyType.LOCATION, description="プロパティタイプ"
    )
    location: EmptyObject = Field(
        default_factory=EmptyObject, description="location設定"
    )




from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr


class FileUploadCreatedBy(BaseModel):
    """file_upload.created_by の最小表現

    type は person | bot | agent のいずれか。
    """

    id: StrictStr
    type: Literal["person", "bot", "agent"]


class FileImportError(BaseModel):
    type: Literal[
        "validation_error",
        "internal_system_error",
        "download_error",
        "upload_error",
    ]
    code: StrictStr
    message: StrictStr
    parameter: StrictStr | None = None
    status_code: StrictInt | None = None


class FileImportResult(BaseModel):
    imported_time: StrictStr
    type: Literal["success", "error"]
    success: dict | None = None
    error: FileImportError | None = None


class FileUploadObject(BaseModel):
    """TypeScript: FileUploadObjectResponse"""

    object: Literal["file_upload"]
    id: StrictStr
    created_time: StrictStr
    created_by: FileUploadCreatedBy
    last_edited_time: StrictStr
    archived: StrictBool
    expiry_time: StrictStr | None
    status: Literal["pending", "uploaded", "expired", "failed"]
    filename: StrictStr | None = None
    content_type: StrictStr | None = None
    content_length: StrictInt | None = None
    upload_url: StrictStr | None = None
    complete_url: StrictStr | None = None
    file_import_result: FileImportResult | None = None
    number_of_parts: dict[str, StrictInt] | None = Field(
        None, description='{"total": n, "sent": m}'
    )



from typing import Literal
from pydantic import Field, StrictStr

from ._base_property import BaseProperty, NotionPropertyType


class LastEditedTimeProperty(BaseProperty[Literal[NotionPropertyType.LAST_EDITED_TIME]]):
    """Notionのlast_edited_timeプロパティ"""

    type: Literal[NotionPropertyType.LAST_EDITED_TIME] = Field(
        NotionPropertyType.LAST_EDITED_TIME, description="プロパティタイプ"
    )

    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")

    def get_display_value(self) -> str:
        """最終編集日時を取得

        Returns:
            StrictStr | None: 最終編集日時（未設定の場合はnull）
        """
        return self.last_edited_time



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class PhoneNumberPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.PHONE_NUMBER]]
):
    """Notionのphone_numberプロパティ設定"""

    type: Literal[NotionPropertyType.PHONE_NUMBER] = Field(
        NotionPropertyType.PHONE_NUMBER, description="プロパティタイプ"
    )
    phone_number: EmptyObject = Field(
        default_factory=EmptyObject, description="phone_number設定"
    )



"""
NotionデータベースオブジェクトのPydanticモデル定義

TypeScript定義: DatabaseObjectResponse
"""

from __future__ import annotations

from typing import Any, Literal

from pydantic import BaseModel, Field, StrictBool, StrictStr
from typing_extensions import TypedDict

from ..models.parent import NotionParent
from ..models.icon import NotionIcon
from ..models.cover import NotionCover
from ..schema.property_configs import DatabasePropertyConfigResponse


class DataSourceReference(TypedDict):
    """データソース参照情報 (2025-09-03で追加)"""

    id: str
    name: str


class NotionDatabase(BaseModel):
    """
    Notionデータベースオブジェクト

    TypeScript: DatabaseObjectResponse

    **重要**: NotionDatabase はデータベース全体のコンテナです。
    データベース内のデータ行を取得するには databases.query() を使用してください。

    用途:
    - データベース全体のメタ情報（タイトル、説明、アイコンなど）
    - データベースのスキーマ定義（properties）
    - データベースの親情報

    取得方法:
    - databases.retrieve(database_id) でデータベース情報を取得
    - databases.create() で新規データベースを作成
    - databases.update() でデータベースを更新

    データ取得:
    - databases.query(database_id) でデータベース内のページ（NotionPage）を取得

    Examples:
        ```python
        # データベース情報を取得
        db = await client.databases.retrieve({"database_id": "abc123"})

        # データベースのタイトルを取得
        if db.title:
            title_text = "".join([t.plain_text for t in db.title])
            print(f"Database: {title_text}")

        # スキーマ定義を確認
        for name, config in db.properties.items():
            print(f"Property: {name}, Type: {config.type}")

        # データベース内のページを取得
        pages = await client.databases.query({"database_id": db.id})
        ```
    """

    object: Literal["database"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="データベースID")
    title: list[Any] = Field(..., description="データベースタイトル（RichText配列）")
    description: list[Any] = Field(
        default_factory=list, description="データベース説明（RichText配列）"
    )
    parent: NotionParent = Field(..., description="親オブジェクト")
    is_inline: StrictBool = Field(False, description="インラインデータベースフラグ")
    in_trash: StrictBool = Field(False, description="ゴミ箱フラグ")
    is_locked: StrictBool = Field(False, description="ロックフラグ")
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    data_sources: list[DataSourceReference] = Field(
        default_factory=list, description="データソース一覧（2025-09-03以降）"
    )
    icon: NotionIcon | None = Field(None, description="アイコン")
    cover: NotionCover | None = Field(None, description="カバー画像")
    url: StrictStr = Field(..., description="データベースURL")
    public_url: StrictStr | None = Field(None, description="パブリックURL")
    properties: dict[str, DatabasePropertyConfigResponse] | None = Field(
        None,
        description="プロパティ設定の辞書 (2025-09-03以降はdataSources.retrieve()で取得)",
    )
    # archived フィールドは TypeScript 定義にない（削除）


class PartialDatabase(BaseModel):
    """部分的なデータベース情報"""

    object: Literal["database"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="データベースID")



from pydantic import BaseModel, Field, StrictBool, StrictStr


class Annotations(BaseModel):
    """NotionのRichTextのアノテーション情報"""

    bold: StrictBool = Field(..., description="太字かどうか")
    italic: StrictBool = Field(..., description="斜体かどうか")
    strikethrough: StrictBool = Field(..., description="取り消し線かどうか")
    underline: StrictBool = Field(..., description="下線かどうか")
    code: StrictBool = Field(..., description="コードかどうか")
    color: StrictStr = Field(..., description="色")



"""
高度なフィルター定義

Formula、Rollup、Verificationフィルター条件。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type FormulaPropertyFilter =
  | { string: TextPropertyFilter }
  | { checkbox: CheckboxPropertyFilter }
  | { number: NumberPropertyFilter }
  | { date: DatePropertyFilter }

type RollupSubfilterPropertyFilter =
  | { rich_text: TextPropertyFilter }
  | { number: NumberPropertyFilter }
  | { checkbox: CheckboxPropertyFilter }
  | { select: SelectPropertyFilter }
  | { multi_select: MultiSelectPropertyFilter }
  | { relation: RelationPropertyFilter }
  | { date: DatePropertyFilter }
  | { people: PeoplePropertyFilter }
  | { files: ExistencePropertyFilter }
  | { status: StatusPropertyFilter }

type RollupPropertyFilter =
  | { any: RollupSubfilterPropertyFilter }
  | { none: RollupSubfilterPropertyFilter }
  | { every: RollupSubfilterPropertyFilter }
  | { date: DatePropertyFilter }
  | { number: NumberPropertyFilter }

type VerificationPropertyStatusFilter = {
  status: "verified" | "expired" | "none"
}
"""

from typing import Literal, TypedDict, Union

from .base_filters import ExistencePropertyFilter
from .date_filters import DatePropertyFilter
from .relation_filters import PeoplePropertyFilter, RelationPropertyFilter
from .value_filters import (
    CheckboxPropertyFilter,
    MultiSelectPropertyFilter,
    NumberPropertyFilter,
    SelectPropertyFilter,
    StatusPropertyFilter,
    TextPropertyFilter,
)


# ============================================================================
# Formula Property Filters
# ============================================================================


class FormulaFilterString(TypedDict):
    """Formula文字列フィルター

    Examples:
        ```python
        filter: FormulaFilterString = {
            "string": {"contains": "test"}
        }
        ```
    """

    string: TextPropertyFilter


class FormulaFilterCheckbox(TypedDict):
    """Formulaチェックボックスフィルター"""

    checkbox: CheckboxPropertyFilter


class FormulaFilterNumber(TypedDict):
    """Formula数値フィルター"""

    number: NumberPropertyFilter


class FormulaFilterDate(TypedDict):
    """Formula日付フィルター"""

    date: DatePropertyFilter


# Matches TypeScript: type FormulaPropertyFilter
FormulaPropertyFilter = Union[
    FormulaFilterString,
    FormulaFilterCheckbox,
    FormulaFilterNumber,
    FormulaFilterDate,
]


# ============================================================================
# Rollup Subfilter Property Filters
# ============================================================================


class RollupSubfilterRichText(TypedDict):
    """Rollupリッチテキストフィルター"""

    rich_text: TextPropertyFilter


class RollupSubfilterNumber(TypedDict):
    """Rollup数値フィルター"""

    number: NumberPropertyFilter


class RollupSubfilterCheckbox(TypedDict):
    """Rollupチェックボックスフィルター"""

    checkbox: CheckboxPropertyFilter


class RollupSubfilterSelect(TypedDict):
    """Rollupセレクトフィルター"""

    select: SelectPropertyFilter


class RollupSubfilterMultiSelect(TypedDict):
    """Rollupマルチセレクトフィルター"""

    multi_select: MultiSelectPropertyFilter


class RollupSubfilterRelation(TypedDict):
    """Rollup Relationフィルター"""

    relation: RelationPropertyFilter


class RollupSubfilterDate(TypedDict):
    """Rollup日付フィルター"""

    date: DatePropertyFilter


class RollupSubfilterPeople(TypedDict):
    """Rollup Peopleフィルター"""

    people: PeoplePropertyFilter


class RollupSubfilterFiles(TypedDict):
    """Rollupファイルフィルター"""

    files: ExistencePropertyFilter


class RollupSubfilterStatus(TypedDict):
    """Rollupステータスフィルター"""

    status: StatusPropertyFilter


# Matches TypeScript: type RollupSubfilterPropertyFilter
RollupSubfilterPropertyFilter = Union[
    RollupSubfilterRichText,
    RollupSubfilterNumber,
    RollupSubfilterCheckbox,
    RollupSubfilterSelect,
    RollupSubfilterMultiSelect,
    RollupSubfilterRelation,
    RollupSubfilterDate,
    RollupSubfilterPeople,
    RollupSubfilterFiles,
    RollupSubfilterStatus,
]


# ============================================================================
# Rollup Property Filters
# ============================================================================


class RollupFilterAny(TypedDict):
    """Rollup いずれかが一致

    Examples:
        ```python
        filter: RollupFilterAny = {
            "any": {"number": {"greater_than": 100}}
        }
        ```
    """

    any: RollupSubfilterPropertyFilter


class RollupFilterNone(TypedDict):
    """Rollup いずれも一致しない"""

    none: RollupSubfilterPropertyFilter


class RollupFilterEvery(TypedDict):
    """Rollup すべてが一致"""

    every: RollupSubfilterPropertyFilter


class RollupFilterDate(TypedDict):
    """Rollup日付フィルター"""

    date: DatePropertyFilter


class RollupFilterNumber(TypedDict):
    """Rollup数値フィルター"""

    number: NumberPropertyFilter


# Matches TypeScript: type RollupPropertyFilter
RollupPropertyFilter = Union[
    RollupFilterAny,
    RollupFilterNone,
    RollupFilterEvery,
    RollupFilterDate,
    RollupFilterNumber,
]


# ============================================================================
# Verification Property Status Filter
# ============================================================================


class VerificationPropertyStatusFilter(TypedDict):
    """検証ステータスフィルター

    Examples:
        ```python
        filter: VerificationPropertyStatusFilter = {"status": "verified"}
        ```
    """

    status: Literal["verified", "expired", "none"]



from typing import Literal
from pydantic import Field, StrictStr

from ._base_property import BaseProperty, NotionPropertyType


class UrlProperty(BaseProperty[Literal[NotionPropertyType.URL]]):
    """Notionのurlプロパティ"""

    type: Literal[NotionPropertyType.URL] = Field(
        NotionPropertyType.URL, description="プロパティタイプ"
    )

    url: StrictStr | None = Field(None, description="URL（設定されていない場合はnull）")

    def get_display_value(self) -> str | None:
        """URLを取得

        Returns:
            StrictStr | None: URL（未設定の場合はnull）
        """
        return self.url



"""
Notionアイコンモデル

Database、Page、DataSourceなどで使用されるアイコンの定義
"""

from __future__ import annotations

from enum import Enum

from pydantic import BaseModel, Field, StrictStr

from .file import ExternalFile, InternalFile
from .primitives import CustomEmoji


class IconType(str, Enum):
    """アイコンタイプ"""

    EMOJI = "emoji"
    EXTERNAL = "external"
    FILE = "file"
    CUSTOM_EMOJI = "custom_emoji"


class NotionIcon(BaseModel):
    """
    Notionのアイコン（絵文字、外部画像、内部ファイル）

    Database、Page、DataSourceで共通使用

    Examples:
        ```python
        # 絵文字アイコン
        emoji_icon = NotionIcon(type=IconType.EMOJI, emoji="📝")

        # 外部画像アイコン
        external_icon = NotionIcon(
            type=IconType.EXTERNAL,
            external=ExternalFile(url="https://example.com/icon.png")
        )

        # 内部ファイルアイコン
        file_icon = NotionIcon(
            type=IconType.FILE,
            file=InternalFile(url="https://notion.so/...", expiry_time="...")
        )
        ```
    """

    type: IconType = Field(..., description="アイコンタイプ")
    emoji: StrictStr | None = Field(None, description="絵文字アイコン")
    external: ExternalFile | None = Field(None, description="外部ファイル")
    file: InternalFile | None = Field(None, description="内部ファイル")
    custom_emoji: CustomEmoji | None = Field(
        None, description="カスタム絵文字（typeがcustom_emojiの場合のみ）"
    )



from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr

from .date_info import DateInfo


class FormulaResult(BaseModel):
    """Notionのformula結果"""

    type: StrictStr = Field(..., description="結果タイプ（string、number、boolean等）")
    string: StrictStr | None = Field(
        None, description="文字列結果（typeがstringの場合）"
    )
    number: StrictInt | StrictFloat | None = Field(
        None, description="数値結果（typeがnumberの場合）"
    )
    boolean: StrictBool | None = Field(
        None, description="真偽値結果（typeがbooleanの場合）"
    )
    date: DateInfo | None = Field(
        None, description="日付結果（typeがdateの場合）"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class LastEditedTimePropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.LAST_EDITED_TIME]]
):
    """Notionのlast_edited_timeプロパティ設定"""

    type: Literal[NotionPropertyType.LAST_EDITED_TIME] = Field(
        NotionPropertyType.LAST_EDITED_TIME, description="プロパティタイプ"
    )
    last_edited_time: EmptyObject = Field(
        default_factory=EmptyObject, description="last_edited_time設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class RichTextPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.RICH_TEXT]]
):
    """Notionのrich_textプロパティ設定"""

    type: Literal[NotionPropertyType.RICH_TEXT] = Field(
        NotionPropertyType.RICH_TEXT, description="プロパティタイプ"
    )
    rich_text: EmptyObject = Field(
        default_factory=EmptyObject, description="rich_text設定"
    )



from ._base_property import BaseProperty, NotionPropertyType
from .button_property import ButtonProperty
from .checkbox_property import CheckboxProperty
from .created_by_property import CreatedByProperty
from .created_time_property import CreatedTimeProperty
from .date_property import DateProperty
from .formula_property import FormulaProperty
from .email_property import EmailProperty
from .files_property import FilesProperty
from .last_edited_time_property import LastEditedTimeProperty
from .last_visited_time_property import LastVisitedTimeProperty
from .last_edited_by_property import LastEditedByProperty
from .multi_select_property import MultiSelectProperty
from .location_property import LocationProperty
from .number_property import NumberProperty
from .people_property import PeopleProperty
from .phone_number_property import PhoneNumberProperty
from .place_property import PlaceProperty
from .rich_text_property import RichTextProperty
from .select_property import SelectProperty
from .status_property import StatusProperty
from .title_property import TitleProperty
from .unique_id_property import UniqueIdProperty
from .url_property import UrlProperty
from .verification_property import VerificationProperty

__all__ = [
    "BaseProperty",
    "NotionPropertyType",
    "ButtonProperty",
    "CheckboxProperty",
    "CreatedByProperty",
    "CreatedTimeProperty",
    "DateProperty",
    "FormulaProperty",
    "EmailProperty",
    "FilesProperty",
    "LastEditedTimeProperty",
    "LastVisitedTimeProperty",
    "LastEditedByProperty",
    "RichTextProperty",
    "StatusProperty",
    "TitleProperty",
    "MultiSelectProperty",
    "UrlProperty",
    "SelectProperty",
    "PeopleProperty",
    "NumberProperty",
    "PhoneNumberProperty",
    "LocationProperty",
    "PlaceProperty",
    "UniqueIdProperty",
    "VerificationProperty",
]



"""
レイアウト系ブロックの定義

Divider, Breadcrumb, TableOfContents, ColumnList, Column, LinkToPage, Table, TableRow
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr

from .base import ApiColor, BaseBlockObject, BlockType
from ..models.rich_text_item import RichTextItem


# ============================================
# Content Models
# ============================================


class EmptyObject(BaseModel):
    """空のオブジェクト"""

    pass


class TableOfContentsContent(BaseModel):
    """目次コンテンツ"""

    color: ApiColor = Field(..., description="カラー設定")


class ColumnContent(BaseModel):
    """カラムコンテンツ"""

    width_ratio: float | None = Field(
        None,
        description="カラムリスト内の全カラムに対するこのカラムの幅の比率（0〜1）。指定しない場合は等幅",
    )


class LinkToPageContent(BaseModel):
    """ページへのリンクコンテンツ"""

    type: Literal["page_id", "database_id", "comment_id"] = Field(
        ..., description="リンクタイプ"
    )
    page_id: StrictStr | None = Field(None, description="ページID")
    database_id: StrictStr | None = Field(None, description="データベースID")
    comment_id: StrictStr | None = Field(None, description="コメントID")


class TableContent(BaseModel):
    """テーブルコンテンツ"""

    has_column_header: StrictBool = Field(..., description="列ヘッダーの有無")
    has_row_header: StrictBool = Field(..., description="行ヘッダーの有無")
    table_width: StrictInt = Field(..., description="テーブルの幅")


class TableRowContent(BaseModel):
    """テーブル行コンテンツ"""

    cells: list[list[RichTextItem]] = Field(..., description="セル配列（2次元配列）")


# ============================================
# Layout Blocks
# ============================================


class DividerBlock(BaseBlockObject):
    """区切り線ブロック"""

    type: Literal[BlockType.DIVIDER] = Field(
        BlockType.DIVIDER, description="ブロックタイプ"
    )
    divider: EmptyObject = Field(
        default_factory=EmptyObject, description="空のコンテンツ"
    )


class BreadcrumbBlock(BaseBlockObject):
    """パンくずリストブロック"""

    type: Literal[BlockType.BREADCRUMB] = Field(
        BlockType.BREADCRUMB, description="ブロックタイプ"
    )
    breadcrumb: EmptyObject = Field(
        default_factory=EmptyObject, description="空のコンテンツ"
    )


class TableOfContentsBlock(BaseBlockObject):
    """目次ブロック"""

    type: Literal[BlockType.TABLE_OF_CONTENTS] = Field(
        BlockType.TABLE_OF_CONTENTS, description="ブロックタイプ"
    )
    table_of_contents: TableOfContentsContent = Field(..., description="目次コンテンツ")


class ColumnListBlock(BaseBlockObject):
    """カラムリストブロック"""

    type: Literal[BlockType.COLUMN_LIST] = Field(
        BlockType.COLUMN_LIST, description="ブロックタイプ"
    )
    column_list: EmptyObject = Field(
        default_factory=EmptyObject, description="空のコンテンツ"
    )


class ColumnBlock(BaseBlockObject):
    """カラムブロック"""

    type: Literal[BlockType.COLUMN] = Field(
        BlockType.COLUMN, description="ブロックタイプ"
    )
    column: ColumnContent = Field(..., description="カラムコンテンツ")


class LinkToPageBlock(BaseBlockObject):
    """ページへのリンクブロック"""

    type: Literal[BlockType.LINK_TO_PAGE] = Field(
        BlockType.LINK_TO_PAGE, description="ブロックタイプ"
    )
    link_to_page: LinkToPageContent = Field(..., description="リンクコンテンツ")


class TableBlock(BaseBlockObject):
    """テーブルブロック"""

    type: Literal[BlockType.TABLE] = Field(
        BlockType.TABLE, description="ブロックタイプ"
    )
    table: TableContent = Field(..., description="テーブルコンテンツ")


class TableRowBlock(BaseBlockObject):
    """テーブル行ブロック"""

    type: Literal[BlockType.TABLE_ROW] = Field(
        BlockType.TABLE_ROW, description="ブロックタイプ"
    )
    table_row: TableRowContent = Field(..., description="テーブル行コンテンツ")



"""
テキスト系ブロックの定義

Paragraph, Heading1/2/3, BulletedListItem, NumberedListItem, Quote, ToDo, Toggle, Template
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictBool

from .base import ApiColor, BaseBlockObject, BlockType
from ..models.rich_text_item import RichTextItem


class ContentWithRichTextAndColor(BaseModel):
    """リッチテキストとカラーを持つコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    color: ApiColor = Field(..., description="カラー設定")


class HeaderContentWithRichTextAndColor(BaseModel):
    """ヘッダー用リッチテキストとカラーを持つコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    color: ApiColor = Field(..., description="カラー設定")
    is_toggleable: StrictBool = Field(..., description="トグル可能かどうか")


# ============================================
# Text Blocks
# ============================================


class ParagraphBlock(BaseBlockObject):
    """段落ブロック"""

    type: Literal[BlockType.PARAGRAPH] = Field(
        BlockType.PARAGRAPH, description="ブロックタイプ"
    )
    paragraph: ContentWithRichTextAndColor = Field(..., description="段落コンテンツ")


class Heading1Block(BaseBlockObject):
    """見出し1ブロック"""

    type: Literal[BlockType.HEADING_1] = Field(
        BlockType.HEADING_1, description="ブロックタイプ"
    )
    heading_1: HeaderContentWithRichTextAndColor = Field(
        ..., description="見出し1コンテンツ"
    )


class Heading2Block(BaseBlockObject):
    """見出し2ブロック"""

    type: Literal[BlockType.HEADING_2] = Field(
        BlockType.HEADING_2, description="ブロックタイプ"
    )
    heading_2: HeaderContentWithRichTextAndColor = Field(
        ..., description="見出し2コンテンツ"
    )


class Heading3Block(BaseBlockObject):
    """見出し3ブロック"""

    type: Literal[BlockType.HEADING_3] = Field(
        BlockType.HEADING_3, description="ブロックタイプ"
    )
    heading_3: HeaderContentWithRichTextAndColor = Field(
        ..., description="見出し3コンテンツ"
    )


class BulletedListItemBlock(BaseBlockObject):
    """箇条書きリストアイテムブロック"""

    type: Literal[BlockType.BULLETED_LIST_ITEM] = Field(
        BlockType.BULLETED_LIST_ITEM, description="ブロックタイプ"
    )
    bulleted_list_item: ContentWithRichTextAndColor = Field(
        ..., description="箇条書きリストアイテムコンテンツ"
    )


class NumberedListItemBlock(BaseBlockObject):
    """番号付きリストアイテムブロック"""

    type: Literal[BlockType.NUMBERED_LIST_ITEM] = Field(
        BlockType.NUMBERED_LIST_ITEM, description="ブロックタイプ"
    )
    numbered_list_item: ContentWithRichTextAndColor = Field(
        ..., description="番号付きリストアイテムコンテンツ"
    )


class QuoteBlock(BaseBlockObject):
    """引用ブロック"""

    type: Literal[BlockType.QUOTE] = Field(
        BlockType.QUOTE, description="ブロックタイプ"
    )
    quote: ContentWithRichTextAndColor = Field(..., description="引用コンテンツ")


class ToDoContent(BaseModel):
    """ToDoコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    color: ApiColor = Field(..., description="カラー設定")
    checked: StrictBool | None = Field(None, description="チェック状態")


class ToDoBlock(BaseBlockObject):
    """ToDoブロック"""

    type: Literal[BlockType.TO_DO] = Field(
        BlockType.TO_DO, description="ブロックタイプ"
    )
    to_do: ToDoContent = Field(..., description="ToDoコンテンツ")


class ToggleBlock(BaseBlockObject):
    """トグルブロック"""

    type: Literal[BlockType.TOGGLE] = Field(
        BlockType.TOGGLE, description="ブロックタイプ"
    )
    toggle: ContentWithRichTextAndColor = Field(..., description="トグルコンテンツ")


class TemplateBlock(BaseBlockObject):
    """テンプレートブロック"""

    type: Literal[BlockType.TEMPLATE] = Field(
        BlockType.TEMPLATE, description="ブロックタイプ"
    )
    template: ContentWithRichTextAndColor = Field(
        ..., description="テンプレートコンテンツ"
    )



from abc import ABC, abstractmethod
from typing import (
    Any,
    Callable,
    Generic,
    TypeVar,
    overload,
    Never,
)
from pydantic import BaseModel
from .responses.page import NotionPage, PropertyType
from .requests.page_requests import CreatePageParameters, UpdatePageParameters
from .requests.property_requests import PropertyRequest

_TProperty = TypeVar("_TProperty", bound=PropertyType)
_TPropertyRequest = TypeVar("_TPropertyRequest", bound=PropertyRequest | Never)
_TDomainValue = TypeVar("_TDomainValue")


class NotionPropertyDescriptor(
    Generic[_TProperty, _TPropertyRequest, _TDomainValue], ABC
):
    """NotionプロパティとDomainモデル間の双方向変換を提供するディスクリプタ.

    このクラスは以下の機能を提供します:
    - Notionプロパティ値からDomain値への変換（parse）
    - Domain値からNotionプロパティリクエストへの変換（build_request）

    Genericsパラメータ:
        _TProperty: Notionプロパティの型（例: TitleProperty, NumberProperty）
        _TPropertyRequest: プロパティリクエストの型（例: TitlePropertyRequest）
            読み取り専用プロパティの場合はNeverを指定
        _TDomainValue: Domainモデルでの値の型（例: str, int, date）

    Attributes:
        notion_name: Notionでのプロパティ名（例: "名前", "Service"）
        parser: NotionプロパティからDomain値への変換関数
        request_builder: Domain値からPropertyRequestへの変換関数

    Examples:
        >>> # Title プロパティのディスクリプタ
        >>> title_field = NotionPropertyDescriptor(
        ...     notion_name="名前",
        ...     parser=lambda p: p.title[0].plain_text if p.title else "",
        ...     request_builder=lambda v: TitlePropertyRequest(
        ...         title=[{"type": "text", "text": {"content": v}}]
        ...     )
        ... )
        >>>
        >>> # Parse: Notion -> Domain
        >>> domain_value: str = title_field.parse(notion_property)
        >>>
        >>> # Build: Domain -> Notion
        >>> request: TitlePropertyRequest = title_field.build_request("新しい名前")
        >>>
        >>> # 読み取り専用プロパティ（Formula等）
        >>> formula_field: NotionPropertyDescriptor[FormulaProperty, Never, float] = (
        ...     NotionPropertyDescriptor(
        ...         notion_name="契約期間(ヶ月)",
        ...         parser=lambda p: p.formula.number or 0,
        ...         request_builder=None,
        ...     )
        ... )
    """

    def __init__(
        self,
        notion_name: str,
        parser: Callable[[_TProperty], _TDomainValue] | None,
        request_builder: Callable[[_TDomainValue], _TPropertyRequest] | None,
    ):
        """NotionPropertyDescriptorを初期化.

        Args:
            notion_name: Notionでのプロパティ名
            parser: NotionプロパティからDomain値への変換関数。
                Noneの場合は書き込み専用（parseメソッド呼び出し不可）
            request_builder: Domain値からPropertyRequestへの変換関数。
                Noneの場合は読み取り専用（build_requestメソッド呼び出し不可）

        Note:
            Formulaなど読み取り専用プロパティの場合、request_builder=Noneを指定し、
            型パラメータは NotionPropertyDescriptor[FormulaProperty, Never, float] のように
            Neverを使用してください。
        """
        self.notion_name = notion_name
        self.parser = parser
        self.request_builder = request_builder

    def parse(self, property: _TProperty) -> _TDomainValue:
        """NotionプロパティからDomain値へ変換.

        Args:
            property: Notionプロパティオブジェクト

        Returns:
            変換されたDomain値

        Raises:
            ValueError: parserがNone（書き込み専用）の場合
        """
        if self.parser is None:
            raise ValueError(
                f"Property '{self.notion_name}' is write-only and cannot parse values"
            )
        return self.parser(property)

    def build_request(self, value: _TDomainValue) -> _TPropertyRequest:
        """Domain値からNotionプロパティリクエストへ変換.

        Args:
            value: Domain値

        Returns:
            Notionプロパティ更新リクエスト

        Raises:
            ValueError: request_builderがNone（読み取り専用）の場合
        """
        if self.request_builder is None:
            raise ValueError(
                f"Property '{self.notion_name}' is read-only and cannot build requests"
            )
        return self.request_builder(value)


@overload
def Field(
    notion_name: str,
    parser: None = None,
    request_builder: None = None,
) -> NotionPropertyDescriptor[Any, Any, Any]: ...


@overload
def Field(
    notion_name: str,
    parser: Callable[[_TProperty], _TDomainValue],
    request_builder: None = None,
) -> NotionPropertyDescriptor[_TProperty, Any, _TDomainValue]: ...


@overload
def Field(
    notion_name: str,
    parser: None = None,
    request_builder: Callable[[_TDomainValue], _TPropertyRequest] = ...,  # type: ignore
) -> NotionPropertyDescriptor[Any, _TPropertyRequest, _TDomainValue]: ...


@overload
def Field(
    notion_name: str,
    parser: Callable[[_TProperty], _TDomainValue],
    request_builder: Callable[[_TDomainValue], _TPropertyRequest],
) -> NotionPropertyDescriptor[_TProperty, _TPropertyRequest, _TDomainValue]: ...


def Field(
    notion_name: str,
    parser: Callable[[_TProperty], _TDomainValue] | None = None,
    request_builder: Callable[[_TDomainValue], _TPropertyRequest] | None = None,
) -> NotionPropertyDescriptor[Any, Any, Any]:
    """NotionPropertyDescriptorを生成するファクトリ関数.

    型安全なプロパティディスクリプタを生成します。
    parser/request_builderの指定パターンに応じて適切な型推論が行われます。

    Args:
        notion_name: Notionでのプロパティ名
        parser: NotionプロパティからDomain値への変換関数（省略可）
        request_builder: Domain値からPropertyRequestへの変換関数（省略可）

    Returns:
        NotionPropertyDescriptor: 生成されたプロパティディスクリプタ

    Examples:
        >>> # 読み書き可能なフィールド
        >>> service_field = Field(
        ...     notion_name="Service",
        ...     parser=lambda p: p.multi_select[0].name if p.multi_select else "",
        ...     request_builder=lambda v: MultiSelectPropertyRequest(
        ...         multi_select=[{"name": v}]
        ...     )
        ... )
        >>>
        >>> # 読み取り専用フィールド（Formula等）
        >>> duration_field = Field(
        ...     notion_name="契約期間(ヶ月)",
        ...     parser=lambda p: p.formula.number or 0,
        ... )
        >>> # 型は NotionPropertyDescriptor[FormulaProperty, Never, float] として推論される
        >>>
        >>> # 書き込み専用フィールド（稀なケース）
        >>> write_only_field = Field(
        ...     notion_name="SomeProperty",
        ...     request_builder=lambda v: SomePropertyRequest(value=v),
        ... )
    """
    if parser is None:
        parser = lambda _: None  # type: ignore
    if request_builder is None:
        request_builder = lambda _: None  # type: ignore
    return NotionPropertyDescriptor(notion_name, parser, request_builder)  # type: ignore


_TDomainModel = TypeVar("_TDomainModel", bound=BaseModel)


class NotionMapper(Generic[_TDomainModel], ABC):
    """NotionページとDomainモデル間の双方向変換を提供する抽象基底クラス.

    このクラスを継承して具体的なMapperを実装することで、
    NotionページとDomainモデル間の変換ロジックを統一的に管理できます。

    Genericsパラメータ:
        _TDomainModel: 変換対象のDomainモデル型（例: ContractRecord）

    Examples:
        >>> class ContractRecordMapper(NotionMapper[ContractRecord]):
        ...     service_field = Field(
        ...         notion_name="Service",
        ...         parser=lambda p: p.multi_select[0].name if p.multi_select else "",
        ...         request_builder=lambda v: MultiSelectPropertyRequest(
        ...             multi_select=[{"name": v}]
        ...         )
        ...     )
        ...
        ...     def to_domain(self, notion_page: NotionPage) -> ContractRecord:
        ...         props = notion_page.properties
        ...         return ContractRecord(
        ...             id=notion_page.id,
        ...             service=self.service_field.parse(props["Service"]),
        ...             # ...
        ...         )
        ...
        ...     def build_update_properties(
        ...         self, model: ContractRecord
        ...     ) -> UpdatePageParameters:
        ...         return UpdatePageParameters(
        ...             page_id=model.id,
        ...             properties={
        ...                 self.service_field.notion_name: self.service_field.build_request(
        ...                     model.service
        ...                 ),
        ...                 # ...
        ...             }
        ...         )
        ...
        ...     def build_create_properties(
        ...         self, model: ContractRecord
        ...     ) -> CreatePageParameters:
        ...         # 実装省略
        ...         pass
    """

    @abstractmethod
    def to_domain(self, notion_page: NotionPage) -> _TDomainModel:
        """NotionページをDomainモデルに変換.

        Args:
            notion_page: Notionページオブジェクト

        Returns:
            変換されたDomainモデル

        Raises:
            NotImplementedError: サブクラスで実装が必要
        """
        raise NotImplementedError

    @abstractmethod
    def build_update_properties(self, model: _TDomainModel) -> UpdatePageParameters:
        """Domainモデルからページ更新パラメータを構築.

        Args:
            model: Domainモデル（id必須）

        Returns:
            Notionページ更新パラメータ

        Raises:
            NotImplementedError: サブクラスで実装が必要
        """
        raise NotImplementedError

    @abstractmethod
    def build_create_properties(
        self, datasource_id: str, model: _TDomainModel
    ) -> CreatePageParameters:
        """Domainモデルからページ作成パラメータを構築.

        Args:
            model: Domainモデル

        Returns:
            Notionページ作成パラメータ

        Raises:
            NotImplementedError: サブクラスで実装が必要
        """
        raise NotImplementedError



"""
NotionデータソースオブジェクトのPydanticモデル定義

TypeScript定義: DataSourceObjectResponse

データソースは、データベース内のビュー/テーブルの構造を定義します。
PageObjectResponse とは異なり、スキーマ定義を持ちます。
"""

from typing import Any, Literal

from pydantic import BaseModel, Field, StrictBool, StrictStr

from ..models.user import PartialUser
from ..models.icon import NotionIcon
from ..models.cover import NotionCover
from ..models.parent import NotionParent
from ..schema.property_configs import DatabasePropertyConfigResponse


class DataSource(BaseModel):
    """
    Notionデータソースオブジェクト

    TypeScript: DataSourceObjectResponse

    **重要**: DataSourceはデータベースのスキーマ定義（メタ情報）です。
    データベースの実際のデータ行（レコード）は NotionPage です。

    用途:
    - データベース内のビューやテーブルの構造を定義
    - properties には DatabasePropertyConfigResponse が入る（スキーマ定義）
    - データ取得には pages API や databases.query() を使用（NotionPage を返す）

    取得方法:
    - dataSources.retrieve(data_source_id) でスキーマ情報を取得
    - databases.query(database_id) はデータ行（NotionPage）を返す

    Examples:
        ```python
        # データソース（スキーマ）を取得
        datasource = await client.dataSources.retrieve({
            "data_source_id": "abc123"
        })

        # properties にはスキーマ定義が入っている
        for name, config in datasource.properties.items():
            print(f"Property: {name}, Type: {config.type}")
        ```
    """

    object: Literal["data_source"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="データソースID")
    title: list[Any] = Field(..., description="データソースタイトル（RichText配列）")
    description: list[Any] = Field(
        default_factory=list, description="データソース説明（RichText配列）"
    )
    parent: NotionParent = Field(..., description="親オブジェクト")
    database_parent: NotionParent = Field(
        ..., description="データベースの親オブジェクト"
    )
    is_inline: StrictBool = Field(False, description="インラインフラグ")
    archived: StrictBool = Field(False, description="アーカイブフラグ")
    in_trash: StrictBool = Field(False, description="ゴミ箱フラグ")
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    created_by: PartialUser = Field(..., description="作成者")
    last_edited_by: PartialUser = Field(..., description="最終編集者")
    properties: dict[str, DatabasePropertyConfigResponse] = Field(
        ..., description="プロパティスキーマ定義"
    )
    icon: NotionIcon | None = Field(None, description="アイコン")
    cover: NotionCover | None = Field(None, description="カバー画像")
    url: StrictStr = Field(..., description="データソースURL")
    public_url: StrictStr | None = Field(None, description="パブリックURL")


class PartialDataSource(BaseModel):
    """部分的なデータソース情報"""

    object: Literal["data_source"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="データソースID")
    properties: dict[str, DatabasePropertyConfigResponse] = Field(
        ..., description="プロパティスキーマ定義"
    )



"""
PropertyItemObjectResponse と PropertyItemListResponse の型定義
"""

from __future__ import annotations

from typing import Literal, Union

from pydantic import BaseModel, Field, StrictBool, StrictStr

from ..models.date_info import DateInfo
from ..models.file import FileWithName
from ..models.rich_text_item import RichTextItem
from ..models.user import PartialUser, User
from ..properties.rollup_property import Rollup
from ..models.formula import FormulaResult
from ..models.unique_id import UniqueId
from ..models.verification import Verification
from ..models.select_option import SelectOption


# 単一プロパティアイテム
class _BasePropertyItem(BaseModel):
    object: Literal["property_item"] = Field("property_item", description="型名")
    id: StrictStr


class NumberPropertyItem(_BasePropertyItem):
    type: Literal["number"]
    number: float | None


class UrlPropertyItem(_BasePropertyItem):
    type: Literal["url"]
    url: StrictStr | None


class SelectPropertyItem(_BasePropertyItem):
    type: Literal["select"]
    select: SelectOption | None


class MultiSelectPropertyItem(_BasePropertyItem):
    type: Literal["multi_select"]
    multi_select: list[SelectOption]


class StatusPropertyItem(_BasePropertyItem):
    type: Literal["status"]
    status: SelectOption | None


class DatePropertyItem(_BasePropertyItem):
    type: Literal["date"]
    date: DateInfo | None


class EmailPropertyItem(_BasePropertyItem):
    type: Literal["email"]
    email: StrictStr | None


class PhoneNumberPropertyItem(_BasePropertyItem):
    type: Literal["phone_number"]
    phone_number: StrictStr | None


class CheckboxPropertyItem(_BasePropertyItem):
    type: Literal["checkbox"]
    checkbox: StrictBool


class FilesPropertyItem(_BasePropertyItem):
    type: Literal["files"]
    files: list[FileWithName]


class CreatedByPropertyItem(_BasePropertyItem):
    type: Literal["created_by"]
    created_by: PartialUser | User


class CreatedTimePropertyItem(_BasePropertyItem):
    type: Literal["created_time"]
    created_time: StrictStr


class LastEditedByPropertyItem(_BasePropertyItem):
    type: Literal["last_edited_by"]
    last_edited_by: PartialUser | User


class LastEditedTimePropertyItem(_BasePropertyItem):
    type: Literal["last_edited_time"]
    last_edited_time: StrictStr


class FormulaPropertyItem(_BasePropertyItem):
    type: Literal["formula"]
    formula: FormulaResult


class ButtonPropertyItem(_BasePropertyItem):
    type: Literal["button"]
    button: dict


class UniqueIdPropertyItem(_BasePropertyItem):
    type: Literal["unique_id"]
    unique_id: UniqueId


class VerificationPropertyItem(_BasePropertyItem):
    type: Literal["verification"]
    verification: Verification | None


class TitlePropertyItem(_BasePropertyItem):
    type: Literal["title"]
    title: RichTextItem


class RichTextPropertyItem(_BasePropertyItem):
    type: Literal["rich_text"]
    rich_text: RichTextItem


class PeoplePropertyItem(_BasePropertyItem):
    type: Literal["people"]
    people: PartialUser | User


class RelationPropertyItem(_BasePropertyItem):
    type: Literal["relation"]
    relation: dict


class RollupPropertyItem(_BasePropertyItem):
    type: Literal["rollup"]
    rollup: Rollup | dict


PropertyItemObject = Union[
    NumberPropertyItem,
    UrlPropertyItem,
    SelectPropertyItem,
    MultiSelectPropertyItem,
    StatusPropertyItem,
    DatePropertyItem,
    EmailPropertyItem,
    PhoneNumberPropertyItem,
    CheckboxPropertyItem,
    FilesPropertyItem,
    CreatedByPropertyItem,
    CreatedTimePropertyItem,
    LastEditedByPropertyItem,
    LastEditedTimePropertyItem,
    FormulaPropertyItem,
    ButtonPropertyItem,
    UniqueIdPropertyItem,
    VerificationPropertyItem,
    TitlePropertyItem,
    RichTextPropertyItem,
    PeoplePropertyItem,
    RelationPropertyItem,
    RollupPropertyItem,
]


class _PropertyItemMeta(BaseModel):
    """Property items pagination meta used by pages.properties.retrieve"""

    type: Literal["property_item"]
    # title/rich_text/people/relation/rollup のときnext_url等を含む
    # 型バリエーションは仕様通りに構築するには複雑なのでここでは next_url と id のみ保持
    next_url: StrictStr | None
    id: StrictStr


class PropertyItemListResponse(BaseModel):
    object: Literal["list"]
    next_cursor: StrictStr | None
    has_more: StrictBool
    type: Literal["property_item"]
    property_item: _PropertyItemMeta
    results: list[PropertyItemObject]



from typing import Literal

from pydantic import Field

from ...models import PartialUser, User
from ._base_property import BaseProperty, NotionPropertyType


class LastEditedByProperty(BaseProperty[Literal[NotionPropertyType.LAST_EDITED_BY]]):
    """Notionのlast_edited_byプロパティ"""

    type: Literal[NotionPropertyType.LAST_EDITED_BY] = Field(
        NotionPropertyType.LAST_EDITED_BY, description="プロパティタイプ"
    )

    last_edited_by: PartialUser | User = Field(..., description="最終編集者情報")

    def get_display_value(self) -> str | None:
        """最終編集者情報を取得

        Returns:
            str | None: 最終編集者名。最終編集者情報が不完全な場合はNone
        """
        name = getattr(self.last_edited_by, "name", None)
        return name if name else None



from typing import Literal

from pydantic import Field, StrictStr

from ._base_property import BaseProperty, NotionPropertyType


class EmailProperty(BaseProperty[Literal[NotionPropertyType.EMAIL]]):
    """Notionのemailプロパティ"""

    type: Literal[NotionPropertyType.EMAIL] = Field(
        NotionPropertyType.EMAIL, description="プロパティタイプ"
    )

    email: StrictStr | None = Field(
        None, description="メールアドレス（未設定の場合はnull）"
    )

    def get_display_value(self) -> str | None:
        """メールアドレスを取得

        Returns:
            StrictStr | None: メールアドレス（未設定の場合はnull）
        """
        return self.email



"""Unit tests for property request classes."""

import pytest
from notion_py_client.requests.property_requests import (
    TitlePropertyRequest,
    RichTextPropertyRequest,
    StatusPropertyRequest,
    SelectPropertyRequest,
    MultiSelectPropertyRequest,
    NumberPropertyRequest,
    DatePropertyRequest,
    CheckboxPropertyRequest,
    UrlPropertyRequest,
    EmailPropertyRequest,
    PhoneNumberPropertyRequest,
    PeoplePropertyRequest,
    FilesPropertyRequest,
    RelationPropertyRequest,
)
from notion_py_client.requests.common import (
    SelectPropertyItemRequest,
    PartialUserObjectRequest,
    DateRequest,
    RelationItemRequest,
)


class TestTitlePropertyRequest:
    """Test TitlePropertyRequest."""

    def test_create_title_request(self):
        """Test creating a title property request."""
        request = TitlePropertyRequest(
            title=[{"type": "text", "text": {"content": "Test Title"}}]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "title" in result
        assert result["title"][0]["text"]["content"] == "Test Title"

    def test_empty_title_request(self):
        """Test creating an empty title request."""
        request = TitlePropertyRequest(title=[])

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "title" in result
        assert result["title"] == []


class TestRichTextPropertyRequest:
    """Test RichTextPropertyRequest."""

    def test_create_rich_text_request(self):
        """Test creating a rich text property request."""
        request = RichTextPropertyRequest(
            rich_text=[{"type": "text", "text": {"content": "Rich text content"}}]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "rich_text" in result
        assert result["rich_text"][0]["text"]["content"] == "Rich text content"


class TestStatusPropertyRequest:
    """Test StatusPropertyRequest."""

    def test_create_status_request_with_name(self):
        """Test creating a status property request with name."""
        request = StatusPropertyRequest(
            status=SelectPropertyItemRequest(name="In Progress")
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "status" in result
        assert result["status"]["name"] == "In Progress"

    def test_create_status_request_with_id(self):
        """Test creating a status property request with ID."""
        request = StatusPropertyRequest(
            status=SelectPropertyItemRequest(id="status-123")
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "status" in result
        assert result["status"]["id"] == "status-123"

    def test_create_none_status_request(self):
        """Test creating a status request to clear status."""
        request = StatusPropertyRequest(status=None)

        result = request.model_dump(by_alias=True, exclude_none=True)

        # When status is None, it should not appear in the result
        assert "status" not in result or result.get("status") is None


class TestSelectPropertyRequest:
    """Test SelectPropertyRequest."""

    def test_create_select_request(self):
        """Test creating a select property request."""
        request = SelectPropertyRequest(
            select=SelectPropertyItemRequest(name="High Priority")
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "select" in result
        assert result["select"]["name"] == "High Priority"


class TestMultiSelectPropertyRequest:
    """Test MultiSelectPropertyRequest."""

    def test_create_multi_select_request(self):
        """Test creating a multi-select property request."""
        request = MultiSelectPropertyRequest(
            multi_select=[
                SelectPropertyItemRequest(name="Tag1"),
                SelectPropertyItemRequest(name="Tag2"),
            ]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "multi_select" in result
        assert len(result["multi_select"]) == 2
        assert result["multi_select"][0]["name"] == "Tag1"
        assert result["multi_select"][1]["name"] == "Tag2"

    def test_empty_multi_select_request(self):
        """Test creating an empty multi-select request."""
        request = MultiSelectPropertyRequest(multi_select=[])

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "multi_select" in result
        assert result["multi_select"] == []


class TestNumberPropertyRequest:
    """Test NumberPropertyRequest."""

    def test_create_number_request(self):
        """Test creating a number property request."""
        request = NumberPropertyRequest(number=42)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "number" in result
        assert result["number"] == 42

    def test_create_float_number_request(self):
        """Test creating a float number property request."""
        request = NumberPropertyRequest(number=3.14)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "number" in result
        assert result["number"] == 3.14

    def test_create_none_number_request(self):
        """Test creating a number request to clear value."""
        request = NumberPropertyRequest(number=None)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "number" not in result or result.get("number") is None


class TestDatePropertyRequest:
    """Test DatePropertyRequest."""

    def test_create_date_request(self):
        """Test creating a date property request."""
        request = DatePropertyRequest(
            date=DateRequest(start="2025-10-09", end=None, time_zone=None)
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "date" in result
        assert result["date"]["start"] == "2025-10-09"

    def test_create_date_range_request(self):
        """Test creating a date range property request."""
        request = DatePropertyRequest(
            date=DateRequest(
                start="2025-10-09", end="2025-10-10", time_zone="Asia/Tokyo"
            )
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "date" in result
        assert result["date"]["start"] == "2025-10-09"
        assert result["date"]["end"] == "2025-10-10"
        assert result["date"]["time_zone"] == "Asia/Tokyo"

    def test_create_none_date_request(self):
        """Test creating a date request to clear value."""
        request = DatePropertyRequest(date=None)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "date" not in result or result.get("date") is None


class TestCheckboxPropertyRequest:
    """Test CheckboxPropertyRequest."""

    def test_create_checkbox_true_request(self):
        """Test creating a checkbox property request (true)."""
        request = CheckboxPropertyRequest(checkbox=True)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "checkbox" in result
        assert result["checkbox"] is True

    def test_create_checkbox_false_request(self):
        """Test creating a checkbox property request (false)."""
        request = CheckboxPropertyRequest(checkbox=False)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "checkbox" in result
        assert result["checkbox"] is False


class TestUrlPropertyRequest:
    """Test UrlPropertyRequest."""

    def test_create_url_request(self):
        """Test creating a URL property request."""
        request = UrlPropertyRequest(url="https://example.com")

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "url" in result
        assert result["url"] == "https://example.com"

    def test_create_none_url_request(self):
        """Test creating a URL request to clear value."""
        request = UrlPropertyRequest(url=None)

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "url" not in result or result.get("url") is None


class TestEmailPropertyRequest:
    """Test EmailPropertyRequest."""

    def test_create_email_request(self):
        """Test creating an email property request."""
        request = EmailPropertyRequest(email="test@example.com")

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "email" in result
        assert result["email"] == "test@example.com"


class TestPhoneNumberPropertyRequest:
    """Test PhoneNumberPropertyRequest."""

    def test_create_phone_number_request(self):
        """Test creating a phone number property request."""
        request = PhoneNumberPropertyRequest(phone_number="+81-90-1234-5678")

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "phone_number" in result
        assert result["phone_number"] == "+81-90-1234-5678"


class TestPeoplePropertyRequest:
    """Test PeoplePropertyRequest."""

    def test_create_people_request(self):
        """Test creating a people property request."""
        request = PeoplePropertyRequest(
            people=[PartialUserObjectRequest(id="user-123")]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "people" in result
        assert len(result["people"]) == 1
        assert result["people"][0]["id"] == "user-123"

    def test_create_multiple_people_request(self):
        """Test creating a people property request with multiple users."""
        request = PeoplePropertyRequest(
            people=[
                PartialUserObjectRequest(id="user-123"),
                PartialUserObjectRequest(id="user-456"),
            ]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "people" in result
        assert len(result["people"]) == 2


class TestFilesPropertyRequest:
    """Test FilesPropertyRequest."""

    def test_create_files_request(self):
        """Test creating a files property request."""
        request = FilesPropertyRequest(
            files=[{"name": "document.pdf", "external": {"url": "https://example.com/doc.pdf"}}]  # type: ignore
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "files" in result
        assert len(result["files"]) == 1


class TestRelationPropertyRequest:
    """Test RelationPropertyRequest."""

    def test_create_relation_request(self):
        """Test creating a relation property request."""
        request = RelationPropertyRequest(relation=[RelationItemRequest(id="page-123")])

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "relation" in result
        assert len(result["relation"]) == 1
        assert result["relation"][0]["id"] == "page-123"

    def test_create_multiple_relations_request(self):
        """Test creating a relation property request with multiple pages."""
        request = RelationPropertyRequest(
            relation=[
                RelationItemRequest(id="page-123"),
                RelationItemRequest(id="page-456"),
            ]
        )

        result = request.model_dump(by_alias=True, exclude_none=True)

        assert "relation" in result
        assert len(result["relation"]) == 2


class TestPropertyRequestSerialization:
    """Test property request serialization for API compatibility."""

    def test_all_requests_serializable(self):
        """Test that all property requests can be serialized."""
        requests = [
            TitlePropertyRequest(title=[{"type": "text", "text": {"content": "Test"}}]),
            StatusPropertyRequest(status=SelectPropertyItemRequest(name="Done")),
            NumberPropertyRequest(number=100),
            DatePropertyRequest(date=DateRequest(start="2025-01-01")),
            CheckboxPropertyRequest(checkbox=True),
        ]

        for req in requests:
            result = req.model_dump(by_alias=True, exclude_none=True)
            assert isinstance(result, dict)
            assert len(result) > 0



"""Page creation and update request types.

Page作成・更新用のリクエストパラメータ型定義。
"""

from typing import Any

from pydantic import BaseModel, StrictBool

from ..models.parent import NotionParent
from .common import IdRequest, PageIconRequest, PageCoverRequest
from .property_requests import PropertyRequest


class CreatePageParameters(BaseModel):
    """Page作成用パラメータ.

    Examples:
        >>> params = CreatePageParameters(
        ...     parent={"database_id": "xxx"},
        ...     properties={
        ...         "Name": {"title": [{"text": {"content": "My Page"}}]},
        ...         "Status": {"select": {"name": "In Progress"}},
        ...     }
        ... )
    """

    parent: NotionParent
    properties: dict[str, PropertyRequest]
    icon: PageIconRequest | None = None
    cover: PageCoverRequest | None = None
    content: list[Any] | None = None  # BlockObjectRequest
    children: list[Any] | None = None  # BlockObjectRequest


class UpdatePageParameters(BaseModel):
    """Page更新用パラメータ.

    Examples:
        >>> params = UpdatePageParameters(
        ...     page_id="page-id-here",
        ...     properties={
        ...         "Status": {"select": {"name": "Done"}},
        ...         "Checkbox": {"checkbox": True},
        ...     }
        ... )
    """

    page_id: IdRequest
    properties: dict[str, PropertyRequest] | None = None
    icon: PageIconRequest | None = None
    cover: PageCoverRequest | None = None
    is_locked: StrictBool | None = None
    archived: StrictBool | None = None
    in_trash: StrictBool | None = None



from typing import Literal

from pydantic import BaseModel, Field

from ..date_info import DateInfo
from ..user import PartialUser, User
from .custom_emoji import CustomEmoji
from .link_mention import LinkMention
from .link_preview import LinkPreviewMention
from .template_mention import TemplateMention


class Mention(BaseModel):
    """RichTextで利用されるメンション情報"""

    type: Literal[
        "user",
        "date",
        "link_preview",
        "link_mention",
        "page",
        "database",
        "template_mention",
        "custom_emoji",
    ] = Field(..., description="メンションタイプ")
    user: PartialUser | User | None = Field(
        None, description="ユーザーメンションの詳細"
    )
    date: DateInfo | None = Field(None, description="日付メンションの詳細")
    link_preview: LinkPreviewMention | None = Field(
        None, description="リンクプレビューメンションの詳細"
    )
    link_mention: LinkMention | None = Field(
        None, description="リンクメンションの詳細"
    )
    page: dict[str, str] | None = Field(None, description="ページメンションの詳細")
    database: dict[str, str] | None = Field(None, description="データベースメンションの詳細")
    template_mention: TemplateMention | None = Field(
        None, description="テンプレートメンションの詳細"
    )
    custom_emoji: CustomEmoji | None = Field(
        None, description="カスタム絵文字メンションの詳細"
    )



"""
基本的なフィルター定義

存在性チェック等の基本的なフィルター条件。
TypedDict を使用して公式 Notion API 仕様に準拠。
"""

from typing import Literal, TypedDict, Union


class ExistenceFilterEmpty(TypedDict):
    """空であることをチェック

    Examples:
        ```python
        filter: ExistenceFilterEmpty = {"is_empty": True}
        ```
    """

    is_empty: Literal[True]


class ExistenceFilterNotEmpty(TypedDict):
    """空でないことをチェック

    Examples:
        ```python
        filter: ExistenceFilterNotEmpty = {"is_not_empty": True}
        ```
    """

    is_not_empty: Literal[True]


# Union type for existence filters
# Matches TypeScript: type ExistencePropertyFilter = { is_empty: true } | { is_not_empty: true }
ExistencePropertyFilter = Union[ExistenceFilterEmpty, ExistenceFilterNotEmpty]



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .primitives import Annotations, Equation, Mention, Text


class RichTextItem(BaseModel):
    """NotionのRichText要素"""

    type: Literal["text", "mention", "equation"] = Field(
        ..., description="RichTextタイプ"
    )
    text: Text | None = Field(None, description="textタイプの内容")
    mention: Mention | None = Field(None, description="mentionタイプの内容")
    equation: Equation | None = Field(None, description="equationタイプの内容")
    annotations: Annotations = Field(..., description="アノテーション情報")
    plain_text: StrictStr = Field(..., description="プレーンテキスト")
    href: StrictStr | None = Field(None, description="リンクURL")

    def get_plain_text(self) -> str:
        """RichTextItemのプレーンテキストを取得"""
        return self.plain_text



"""
Relation/Peopleフィルター定義

RelationプロパティとPeopleプロパティのフィルター条件。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type PeoplePropertyFilter =
  | { contains: IdRequest }
  | { does_not_contain: IdRequest }
  | ExistencePropertyFilter

type RelationPropertyFilter =
  | { contains: IdRequest }
  | { does_not_contain: IdRequest }
  | ExistencePropertyFilter
"""

from typing import TypedDict, Union

from .base_filters import ExistencePropertyFilter


# ============================================================================
# People Property Filters
# ============================================================================


class PeopleFilterContains(TypedDict):
    """Peopleが含む

    Examples:
        ```python
        filter: PeopleFilterContains = {"contains": "user-id-123"}
        ```
    """

    contains: str


class PeopleFilterDoesNotContain(TypedDict):
    """Peopleが含まない"""

    does_not_contain: str


# Matches TypeScript: type PeoplePropertyFilter
PeoplePropertyFilter = Union[
    PeopleFilterContains,
    PeopleFilterDoesNotContain,
    ExistencePropertyFilter,
]


# ============================================================================
# Relation Property Filters
# ============================================================================


class RelationFilterContains(TypedDict):
    """Relationが含む

    Examples:
        ```python
        filter: RelationFilterContains = {"contains": "page-id-123"}
        ```
    """

    contains: str


class RelationFilterDoesNotContain(TypedDict):
    """Relationが含まない"""

    does_not_contain: str


# Matches TypeScript: type RelationPropertyFilter
RelationPropertyFilter = Union[
    RelationFilterContains,
    RelationFilterDoesNotContain,
    ExistencePropertyFilter,
]



from typing import Literal
from pydantic import Field, StrictBool

from ._base_property import BaseProperty, NotionPropertyType


class CheckboxProperty(BaseProperty[Literal[NotionPropertyType.CHECKBOX]]):
    """Notionのcheckboxプロパティ"""

    type: Literal[NotionPropertyType.CHECKBOX] = Field(
        NotionPropertyType.CHECKBOX, description="プロパティタイプ"
    )

    checkbox: StrictBool = Field(False, description="チェックボックスの状態")

    def get_display_value(self) -> bool:
        """チェックボックスの状態を取得

        Returns:
            StrictBool | None: チェックボックスの状態（True or False）
        """
        return self.checkbox



"""Integration tests for Notion API client.

These tests require actual Notion API credentials and will make real API calls.
Set the NOTION_API_TOKEN environment variable to run these tests.

Usage:
    export NOTION_API_TOKEN="your_api_key"
    pytest test/integration/test_client.py
"""

import os
import pytest
from notion_py_client import NotionAsyncClient
from notion_py_client.requests.common import DateRequest


# Skip integration tests if NOTION_API_TOKEN is not set
pytestmark = pytest.mark.skipif(
    not os.getenv("NOTION_API_TOKEN"),
    reason="NOTION_API_TOKEN environment variable not set",
)


@pytest.fixture
def notion_client() -> NotionAsyncClient:
    """Create NotionAsyncClient with API key from environment."""

    api_key = os.getenv("NOTION_API_TOKEN")
    return NotionAsyncClient(auth=api_key)  # type: ignore


@pytest.fixture
def test_database_id():
    """Get test database ID from environment.

    Set TEST_DATABASE_ID environment variable to use a specific database.
    """
    return os.getenv("TEST_DATABASE_ID")


@pytest.fixture
def test_datasource_id():
    """Get test datasource ID from environment.

    Set TEST_DATASOURCE_ID environment variable to use a specific datasource.
    """
    return os.getenv("TEST_DATASOURCE_ID")


class TestDataSourcesAPI:
    """Integration tests for DataSources API."""

    @pytest.mark.asyncio
    async def test_query_datasource(self, notion_client, test_datasource_id):
        """Test querying a datasource."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, page_size=10
        )

        assert response is not None
        assert hasattr(response, "results")
        assert len(response.results) > 0

        # Verify response structure
        assert hasattr(response, "has_more")
        assert hasattr(response, "next_cursor")
        assert response.object == "list"

        # Verify first result is a NotionPage
        from notion_py_client.responses.page import NotionPage

        first_result = response.results[0]
        assert isinstance(first_result, NotionPage)
        assert hasattr(first_result, "id")
        assert hasattr(first_result, "properties")

    @pytest.mark.asyncio
    async def test_query_with_filter(self, notion_client, test_datasource_id):
        """Test querying a datasource with filters."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        # Example filter for member resource schema (multi-select ProjectRange)
        filter_obj: dict = {
            "property": "ProjectRange",
            "multi_select": {"is_not_empty": True},
        }

        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, filter=filter_obj, page_size=5
        )

        assert response is not None
        assert hasattr(response, "results")
        # Results may be empty if no matching records
        assert isinstance(response.results, list)

    @pytest.mark.asyncio
    async def test_query_with_compound_filter(self, notion_client, test_datasource_id):
        """Test querying with compound AND filter."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        from notion_py_client.filters import create_and_filter

        # Create compound filter (adjust property names to match your datasource)
        filter_obj = create_and_filter(
            {"property": "Assignee", "people": {"is_not_empty": True}},
            {"property": "ProjectRange", "multi_select": {"is_not_empty": True}},
        )

        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id,
            filter=filter_obj,  # type: ignore
            page_size=10,
        )

        assert response is not None
        assert isinstance(response.results, list)

    @pytest.mark.asyncio
    async def test_query_with_sorts(self, notion_client, test_datasource_id):
        """Test querying with sort parameters."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        # Sort by created_time descending
        sorts = [{"timestamp": "created_time", "direction": "descending"}]

        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, sorts=sorts, page_size=5
        )

        assert response is not None
        assert len(response.results) > 0

        # Verify results are sorted (most recent first)
        if len(response.results) >= 2:
            first_time = response.results[0].created_time
            second_time = response.results[1].created_time
            assert first_time >= second_time

    @pytest.mark.asyncio
    async def test_query_pagination(self, notion_client, test_datasource_id):
        """Test pagination with start_cursor."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        # Get first page
        first_response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, page_size=2
        )

        assert first_response is not None

        # If has_more is True, fetch next page
        if first_response.has_more:
            second_response = await notion_client.dataSources.query(
                data_source_id=test_datasource_id,
                page_size=2,
                start_cursor=first_response.next_cursor,
            )

            assert second_response is not None
            assert len(second_response.results) > 0

            # Verify different results
            first_ids = {p.id for p in first_response.results}
            second_ids = {p.id for p in second_response.results}
            assert first_ids.isdisjoint(second_ids)

    @pytest.mark.asyncio
    async def test_retrieve_datasource(self, notion_client, test_datasource_id):
        """Test retrieving datasource schema."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        datasource = await notion_client.dataSources.retrieve(
            data_source_id=test_datasource_id
        )

        assert datasource is not None
        assert hasattr(datasource, "id")
        assert hasattr(datasource, "properties")
        assert datasource.id == test_datasource_id
        assert isinstance(datasource.properties, dict)
        assert len(datasource.properties) > 0


class TestPagesAPI:
    """Integration tests for Pages API."""

    @pytest.mark.asyncio
    async def test_create_and_retrieve_page(self, notion_client, test_datasource_id):
        """Test creating and retrieving a page."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        from datetime import date

        from notion_py_client.requests.common import SelectPropertyItemRequest
        from notion_py_client.requests.page_requests import CreatePageParameters
        from notion_py_client.requests.property_requests import (
            DatePropertyRequest,
            MultiSelectPropertyRequest,
            PeoplePropertyRequest,
            TitlePropertyRequest,
        )

        # Create a new page
        params = CreatePageParameters(
            parent={"type": "data_source_id", "data_source_id": test_datasource_id},
            properties={
                "Assignee": PeoplePropertyRequest(people=[]),
                "DateRange": DatePropertyRequest(
                    date=DateRequest(
                        start=date.today().isoformat(), end=date.today().isoformat()
                    )
                ),
                "ProjectRange": MultiSelectPropertyRequest(
                    multi_select=[SelectPropertyItemRequest(name="2025/01")]
                ),
            },
        )

        created_page = await notion_client.pages.create(params=params)

        assert created_page is not None
        assert created_page.id is not None
        assert created_page.object == "page"

        # Retrieve the created page
        from notion_py_client.api_types import RetrievePageParameters

        retrieve_params: RetrievePageParameters = {"page_id": created_page.id}
        retrieved_page = await notion_client.pages.retrieve(params=retrieve_params)

        assert retrieved_page is not None
        assert retrieved_page.id == created_page.id
        assert retrieved_page.object == "page"

    @pytest.mark.asyncio
    async def test_update_page(self, notion_client, test_datasource_id):
        """Test updating a page."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        from datetime import date

        from notion_py_client.requests.common import SelectPropertyItemRequest
        from notion_py_client.requests.page_requests import (
            CreatePageParameters,
            UpdatePageParameters,
        )
        from notion_py_client.requests.property_requests import (
            DatePropertyRequest,
            MultiSelectPropertyRequest,
            PeoplePropertyRequest,
            TitlePropertyRequest,
        )

        # First create a page
        create_params = CreatePageParameters(
            parent={"type": "data_source_id", "data_source_id": test_datasource_id},
            properties={
                "Assignee": PeoplePropertyRequest(people=[]),
                "DateRange": DatePropertyRequest(
                    date=DateRequest(
                        start=date.today().isoformat(), end=date.today().isoformat()
                    )
                ),
                "ProjectRange": MultiSelectPropertyRequest(
                    multi_select=[SelectPropertyItemRequest(name="2025/01")]
                ),
            },
        )

        created_page = await notion_client.pages.create(params=create_params)

        # Update the page
        update_params = UpdatePageParameters(
            page_id=created_page.id,
            properties={
                "ProjectRange": MultiSelectPropertyRequest(
                    multi_select=[SelectPropertyItemRequest(name="2025/02")]
                ),
            },
        )

        updated_page = await notion_client.pages.update(params=update_params)

        assert updated_page is not None
        assert updated_page.id == created_page.id

        # Verify the multi-select property was updated
        project_prop = updated_page.properties.get("ProjectRange")
        assert project_prop is not None
        from notion_py_client.responses.property_types import MultiSelectProperty

        if isinstance(project_prop, MultiSelectProperty):
            values = {item.name for item in project_prop.multi_select}
            assert "2025/02" in values

    @pytest.mark.skip(reason="Create operations affect Notion workspace")
    @pytest.mark.asyncio
    async def test_create_page_with_multiple_properties(
        self, notion_client, test_datasource_id
    ):
        """Test creating a page with various property types."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        from datetime import date

        from notion_py_client.requests.common import SelectPropertyItemRequest
        from notion_py_client.requests.page_requests import CreatePageParameters
        from notion_py_client.requests.property_requests import (
            DatePropertyRequest,
            MultiSelectPropertyRequest,
            NumberPropertyRequest,
            PeoplePropertyRequest,
            RichTextPropertyRequest,
            TitlePropertyRequest,
        )

        params = CreatePageParameters(
            parent={"type": "database_id", "database_id": test_datasource_id},
            properties={
                "Name": TitlePropertyRequest(
                    title=[{"type": "text", "text": {"content": "Multi-Property Test"}}]
                ),
                "Summary": RichTextPropertyRequest(
                    rich_text=[{"type": "text", "text": {"content": "Test notes"}}]
                ),
                "Capacity": NumberPropertyRequest(number=80),
                "Assignee": PeoplePropertyRequest(people=[]),
                "DateRange": DatePropertyRequest(
                    date=DateRequest(
                        start=date.today().isoformat(),
                        end=date.today().isoformat(),
                    )
                ),
                "ProjectRange": MultiSelectPropertyRequest(
                    multi_select=[
                        SelectPropertyItemRequest(name="2025/01"),
                        SelectPropertyItemRequest(name="2025/02"),
                    ]
                ),
            },
        )

        page = await notion_client.pages.create(params=params)

        assert page is not None
        assert page.id is not None
        assert "Name" in page.properties


class TestBlocksAPI:
    """Integration tests for Blocks API."""

    @pytest.mark.asyncio
    async def test_retrieve_block_children(self, notion_client, test_datasource_id):
        """Test retrieving block children from a page."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        # First get a page from the datasource
        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, page_size=1
        )

        if len(response.results) == 0:
            pytest.skip("No pages in datasource")

        page_id = response.results[0].id

        # Retrieve block children
        blocks_response = await notion_client.blocks.children.list(
            block_id=page_id, page_size=10
        )

        assert blocks_response is not None
        assert "results" in blocks_response
        assert isinstance(blocks_response["results"], list)

    @pytest.mark.skip(reason="Append operations affect Notion workspace")
    @pytest.mark.asyncio
    async def test_append_block_children(self, notion_client, test_datasource_id):
        """Test appending block children to a page."""
        if not test_datasource_id:
            pytest.skip("TEST_DATASOURCE_ID not set")

        from notion_py_client.requests.page_requests import CreatePageParameters
        from notion_py_client.requests.property_requests import TitlePropertyRequest

        # Create a new page to add blocks to
        params = CreatePageParameters(
            parent={"type": "database_id", "database_id": test_datasource_id},
            properties={
                "Name": TitlePropertyRequest(
                    title=[{"type": "text", "text": {"content": "Page with Blocks"}}]
                ),
            },
        )

        page = await notion_client.pages.create(params=params)

        # Append paragraph block
        from notion_py_client.api_types import AppendBlockChildrenParameters

        append_params: AppendBlockChildrenParameters = {
            "block_id": page.id,
            "children": [
                {
                    "object": "block",
                    "type": "paragraph",
                    "paragraph": {
                        "rich_text": [
                            {
                                "type": "text",
                                "text": {"content": "This is a test paragraph."},
                            }
                        ]
                    },
                }
            ],
        }

        result = await notion_client.blocks.children.append(params=append_params)

        assert result is not None
        assert "results" in result
        assert len(result["results"]) > 0


class TestDatabasesAPI:
    """Integration tests for Databases API (legacy)."""

    @pytest.mark.asyncio
    async def test_retrieve_database(self, notion_client, test_database_id):
        """Test retrieving a database."""
        if not test_database_id:
            pytest.skip("TEST_DATABASE_ID not set")

        from notion_py_client.api_types import RetrieveDatabaseParameters

        params: RetrieveDatabaseParameters = {"database_id": test_database_id}
        database = await notion_client.databases.retrieve(params=params)

        assert database is not None
        assert hasattr(database, "id")
        # Normalize UUID comparison (remove hyphens)
        assert database.id.replace("-", "") == test_database_id.replace("-", "")
        assert hasattr(database, "data_sources")

    @pytest.mark.skip(
        reason="databases.query() method is legacy and may not be supported"
    )
    @pytest.mark.asyncio
    async def test_query_database(self, notion_client, test_database_id):
        """Test querying a database (legacy endpoint)."""
        if not test_database_id:
            pytest.skip("TEST_DATABASE_ID not set")

        from notion_py_client.api_types import QueryDatabaseParameters

        params: QueryDatabaseParameters = {
            "database_id": test_database_id,
            "page_size": 5,
        }

        response = await notion_client.databases.query(params=params)

        assert response is not None
        assert hasattr(response, "results")
        assert isinstance(response.results, list)


class TestUsersAPI:
    """Integration tests for Users API."""

    @pytest.mark.asyncio
    async def test_list_users(self, notion_client):
        """Test listing users."""
        response = await notion_client.users.list(page_size=10)

        assert response is not None
        assert hasattr(response, "results")
        assert isinstance(response.results, list)
        assert response.object == "list"

        # Verify user structure
        if len(response.results) > 0:
            user = response.results[0]
            assert hasattr(user, "id")
            assert hasattr(user, "object")
            assert user.object == "user"

    @pytest.mark.asyncio
    async def test_retrieve_bot_user(self, notion_client):
        """Test retrieving bot user details."""
        bot_user = await notion_client.users.me()

        assert bot_user is not None
        assert "object" in bot_user
        assert bot_user["object"] == "user"
        assert "type" in bot_user
        assert bot_user["type"] == "bot"


class TestSearchAPI:
    """Integration tests for Search API."""

    @pytest.mark.skip(reason="search.search() method not yet implemented")
    @pytest.mark.asyncio
    async def test_search_pages(self, notion_client):
        """Test search functionality for pages."""
        from notion_py_client.api_types import SearchParameters

        params: SearchParameters = {
            "query": "test",
            "filter": {"property": "object", "value": "page"},
            "page_size": 5,
        }

        response = await notion_client.search.search(params=params)

        assert response is not None
        assert hasattr(response, "results")
        assert isinstance(response.results, list)
        assert response.object == "list"

    @pytest.mark.skip(
        reason="search.search() method is legacy and may not be supported"
    )
    @pytest.mark.asyncio
    async def test_search_databases(self, notion_client):
        """Test search functionality for databases."""
        from notion_py_client.api_types import SearchParameters

        params: SearchParameters = {
            "filter": {"property": "object", "value": "database"},
            "page_size": 5,
        }

        response = await notion_client.search.search(params=params)

        assert response is not None
        assert hasattr(response, "results")
        assert isinstance(response.results, list)

    @pytest.mark.skip(reason="search.search() method not yet implemented")
    @pytest.mark.asyncio
    async def test_search_with_sort(self, notion_client):
        """Test search with sort parameter."""
        from notion_py_client.api_types import SearchParameters

        params: SearchParameters = {
            "query": "",
            "sort": {"direction": "descending", "timestamp": "last_edited_time"},
            "page_size": 5,
        }

        response = await notion_client.search.search(params=params)

        assert response is not None
        assert hasattr(response, "results")

        # Verify results are sorted if there are multiple
        if len(response.results) >= 2:
            # Results should be in descending order of last_edited_time
            for i in range(len(response.results) - 1):
                current = response.results[i]
                next_item = response.results[i + 1]
                if hasattr(current, "last_edited_time") and hasattr(
                    next_item, "last_edited_time"
                ):
                    assert current.last_edited_time >= next_item.last_edited_time



from typing import Literal

from pydantic import Field

from ...models import FileWithName
from ._base_property import BaseProperty, NotionPropertyType


class FilesProperty(BaseProperty[Literal[NotionPropertyType.FILES]]):
    """Notionのfilesプロパティ"""

    type: Literal[NotionPropertyType.FILES] = Field(
        NotionPropertyType.FILES, description="プロパティタイプ"
    )

    files: list[FileWithName] = Field(
        default_factory=list, description="ファイル情報のリスト"
    )

    def get_display_value(self) -> str | None:
        """ファイル名のリストを取得

        Returns:
            str | None: ファイル名をカンマ区切りで連結した文字列。ファイルがない場合はNone
        """
        if len(self.files) == 0:
            return None
        return ", ".join(file.name for file in self.files)



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from ._base_config import BasePropertyConfig
from ....properties.base_properties._base_property import NotionPropertyType


class StatusOptionConfig(BaseModel):
    """statusのオプション定義"""

    id: StrictStr = Field(..., description="ステータスオプションID")
    name: StrictStr = Field(..., description="オプション名")
    color: StrictStr = Field(..., description="表示カラー")
    description: StrictStr | None = Field(None, description="オプション説明")


class StatusGroupConfig(BaseModel):
    """statusのグループ定義"""

    id: StrictStr = Field(..., description="ステータスグループID")
    name: StrictStr = Field(..., description="グループ名")
    color: StrictStr = Field(..., description="表示カラー")
    option_ids: list[StrictStr] = Field(..., description="このグループ内のオプションID")


class StatusConfig(BaseModel):
    """status設定"""

    options: list[StatusOptionConfig] = Field(
        default_factory=list, description="ステータスオプション一覧"
    )
    groups: list[StatusGroupConfig] = Field(
        default_factory=list, description="ステータスグループ一覧"
    )


class StatusPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.STATUS]]
):
    """Notionのstatusプロパティ設定"""

    type: Literal[NotionPropertyType.STATUS] = Field(
        NotionPropertyType.STATUS, description="プロパティタイプ"
    )
    status: StatusConfig = Field(default_factory=StatusConfig, description="status設定")



from .property_configs import (
    BasePropertyConfig,
    ButtonPropertyConfig,
    CheckboxPropertyConfig,
    CreatedByPropertyConfig,
    CreatedTimePropertyConfig,
    DatePropertyConfig,
    EmailPropertyConfig,
    FilesPropertyConfig,
    FormulaPropertyConfig,
    LastEditedByPropertyConfig,
    LastEditedTimePropertyConfig,
    MultiSelectOptions,
    MultiSelectPropertyConfig,
    NumberPropertyConfig,
    PeoplePropertyConfig,
    PhoneNumberPropertyConfig,
    PropertyConfig,
    RelationConfig,
    RelationDualConfig,
    RelationPropertyConfig,
    RelationSingleConfig,
    RichTextPropertyConfig,
    RollupPropertyConfig,
    RollupSettings,
    SelectOptionConfig,
    SelectOptions,
    SelectPropertyConfig,
    StatusPropertyConfig,
    TitlePropertyConfig,
    UniqueIdPropertyConfig,
    UrlPropertyConfig,
    VerificationPropertyConfig,
)

__all__ = [
    "BasePropertyConfig",
    "ButtonPropertyConfig",
    "CheckboxPropertyConfig",
    "CreatedByPropertyConfig",
    "CreatedTimePropertyConfig",
    "DatePropertyConfig",
    "EmailPropertyConfig",
    "FilesPropertyConfig",
    "FormulaPropertyConfig",
    "LastEditedByPropertyConfig",
    "LastEditedTimePropertyConfig",
    "SelectOptionConfig",
    "SelectOptions",
    "MultiSelectOptions",
    "MultiSelectPropertyConfig",
    "NumberPropertyConfig",
    "PeoplePropertyConfig",
    "PhoneNumberPropertyConfig",
    "RelationConfig",
    "RelationSingleConfig",
    "RelationDualConfig",
    "RelationPropertyConfig",
    "RichTextPropertyConfig",
    "RollupPropertyConfig",
    "RollupSettings",
    "SelectPropertyConfig",
    "StatusPropertyConfig",
    "TitlePropertyConfig",
    "UniqueIdPropertyConfig",
    "UrlPropertyConfig",
    "VerificationPropertyConfig",
    "PropertyConfig",
]



"""Pytest configuration and fixtures."""

import pytest

from notion_py_client.notion_client import NotionAsyncClient
from notion_py_client.properties.base_properties.multi_select_property import (
    MultiSelectProperty,
)
from notion_py_client.properties.base_properties.people_property import PeopleProperty
from notion_py_client.requests.page_requests import (
    CreatePageParameters,
    UpdatePageParameters,
)
from notion_py_client.helper import (
    NotionMapper,
    NotionPropertyDescriptor,
    Field as NotionField,
)
from notion_py_client.properties.base_properties._base_property import (
    NotionPropertyType,
)
from notion_py_client.properties.base_properties.date_property import DateProperty
from notion_py_client.requests.common import (
    DateRequest,
    PartialUserObjectRequest,
    SelectPropertyItemRequest,
)
from notion_py_client.requests.property_requests import (
    DatePropertyRequest,
    MultiSelectPropertyRequest,
    PeoplePropertyRequest,
)
from notion_py_client.responses.page import NotionPage

from datetime import date
from typing import Annotated

from pydantic import BaseModel, Field, StrictStr, StringConstraints
from dotenv import load_dotenv

from notion_py_client.utils import is_full_page

load_dotenv()  # Load environment variables from .env file


class DateRange(BaseModel):
    start_date: date = Field(..., description="The start date of the date range")
    end_date: date = Field(..., description="The end date of the date range")
    time_zone: StrictStr | None = Field(
        None, description="The time zone of the date range"
    )


class MemberResource(BaseModel):
    id: StrictStr = Field(..., description="The ID of the member resource")
    assignee: StrictStr | None = Field(
        None, description="The assignee of the member resource"
    )
    date_range: DateRange = Field(
        ...,
        description="The date range of the member resource",
        examples=[
            {
                "start_date": "2025-01-01",
                "end_date": "2025-02-01",
                "time_zone": "Asia/Tokyo",
            }
        ],
    )
    project_range: list[
        Annotated[StrictStr, StringConstraints(pattern=r"^\d{4}/\d{2}$")]
    ] = Field(
        ...,
        description="The project range of the member resource",
        examples=[["2025/01", "2025/02"]],
    )


class MockMapper(NotionMapper[MemberResource]):
    member_field: NotionPropertyDescriptor[
        PeopleProperty, PeoplePropertyRequest, str | None
    ] = NotionField(
        notion_name="Assignee",
        parser=lambda p: (
            getattr(p.people[0], "name", None)
            if p.people and len(p.people) > 0
            else None
        ),
        request_builder=lambda v: (
            PeoplePropertyRequest(people=[PartialUserObjectRequest(id=v)])
            if v is not None
            else PeoplePropertyRequest(people=[])
        ),
    )
    date_range_field: NotionPropertyDescriptor[
        DateProperty, DatePropertyRequest, DateRange
    ] = NotionField(
        notion_name="DateRange",
        parser=lambda p: (
            DateRange(
                start_date=p.get_start_date(),
                end_date=p.get_end_date(),
                time_zone=p.date.time_zone if p.date and p.date.time_zone else None,
            )
        ),
        request_builder=lambda v: DatePropertyRequest(
            date=DateRequest(
                start=v.start_date.isoformat(),
                end=v.end_date.isoformat(),
                time_zone=v.time_zone,
            )
        ),
    )
    project_range_field: NotionPropertyDescriptor[
        MultiSelectProperty, MultiSelectPropertyRequest, list[str]
    ] = NotionField(
        notion_name="ProjectRange",
        parser=lambda p: (
            [item.name for item in p.multi_select] if len(p.multi_select) > 0 else []
        ),
        request_builder=lambda v: (
            MultiSelectPropertyRequest(
                multi_select=[SelectPropertyItemRequest(name=name) for name in v]
            )
            if v
            else MultiSelectPropertyRequest(multi_select=[])
        ),
    )

    def to_domain(self, notion_page: NotionPage) -> MemberResource:
        properties = notion_page.properties
        member_property = properties[self.member_field.notion_name]
        if not member_property.type == NotionPropertyType.PEOPLE:
            raise ValueError(
                f"Expected PeopleProperty for '{self.member_field.notion_name}', got {member_property.type}"
            )
        date_property = properties[self.date_range_field.notion_name]
        if not date_property.type == NotionPropertyType.DATE:
            raise ValueError(
                f"Expected DateProperty for '{self.date_range_field.notion_name}', got {date_property.type}"
            )
        project_property = properties[self.project_range_field.notion_name]
        if not project_property.type == NotionPropertyType.MULTI_SELECT:
            raise ValueError(
                f"Expected MultiSelectProperty for '{self.project_range_field.notion_name}', got {project_property.type}"
            )
        date_range = self.date_range_field.parse(date_property)
        if date_range is None:
            raise ValueError("Date range lacking end date is not supported")
        return MemberResource(
            id=notion_page.id,
            assignee=self.member_field.parse(member_property),
            date_range=date_range,
            project_range=self.project_range_field.parse(project_property),
        )

    def build_create_properties(
        self, datasource_id: str, model: MemberResource
    ) -> CreatePageParameters: ...

    def build_update_properties(
        self, model: MemberResource
    ) -> UpdatePageParameters: ...


@pytest.fixture
def mock_notion_page():
    """Mock NotionPage for testing."""
    # TODO: Add mock NotionPage data
    pass


@pytest.fixture
def mock_database():
    """Mock NotionDatabase for testing."""
    # TODO: Add mock NotionDatabase data
    pass


@pytest.fixture
def mock_datasource():
    """Mock DataSource for testing."""
    # TODO: Add mock DataSource data
    notion_client = NotionAsyncClient(auth="fake_api_key")
    response = notion_client.dataSources.query(
        data_source_id="fake_datasource_id",
        page_size=1,
        filter={
            "property": "Status",
            "select": {"equals": "Active"},
        },
    )


@pytest.fixture
def mock_notion_mapper():
    """Mock NotionMapper for testing."""
    return MockMapper()



from typing import Literal

from pydantic import Field
from ._base_property import BaseProperty, NotionPropertyType


class ButtonProperty(BaseProperty[Literal[NotionPropertyType.BUTTON]]):
    """Notionのbuttonプロパティ"""

    type: Literal[NotionPropertyType.BUTTON] = Field(
        NotionPropertyType.BUTTON, description="プロパティタイプ"
    )

    # buttonプロパティは通常、値を持たない
    pass

    def get_value(self) -> None:
        """
        button プロパティは値を持たないため常にNoneを返す

        Returns:
            None: buttonプロパティはアクション用で、データ値を持たない

        Note:
            - buttonプロパティはNotionでクリックアクションを実行するためのものです
        """
        return None

    def get_display_value(self) -> None:
        """UI表示用の値（buttonは値を持たないため常にNone）。"""
        return None



"""
タイムスタンプフィルター定義

created_time、last_edited_timeに対するフィルター条件。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type TimestampCreatedTimeFilter = {
  created_time: DatePropertyFilter
  timestamp: "created_time"
  type?: "created_time"
}

type TimestampLastEditedTimeFilter = {
  last_edited_time: DatePropertyFilter
  timestamp: "last_edited_time"
  type?: "last_edited_time"
}

type TimestampFilter =
  | TimestampCreatedTimeFilter
  | TimestampLastEditedTimeFilter
"""

from typing import Literal, NotRequired, TypedDict, Union

from .date_filters import DatePropertyFilter


# ============================================================================
# Timestamp Filters
# ============================================================================


class TimestampCreatedTimeFilter(TypedDict):
    """作成日時タイムスタンプフィルター

    Examples:
        ```python
        filter: TimestampCreatedTimeFilter = {
            "timestamp": "created_time",
            "created_time": {"past_week": {}}
        }
        ```
    """

    timestamp: Literal["created_time"]
    created_time: DatePropertyFilter
    type: NotRequired[Literal["created_time"]]


class TimestampLastEditedTimeFilter(TypedDict):
    """最終編集日時タイムスタンプフィルター

    Examples:
        ```python
        filter: TimestampLastEditedTimeFilter = {
            "timestamp": "last_edited_time",
            "last_edited_time": {"after": "2025-01-01"}
        }
        ```
    """

    timestamp: Literal["last_edited_time"]
    last_edited_time: DatePropertyFilter
    type: NotRequired[Literal["last_edited_time"]]


# Matches TypeScript: type TimestampFilter
TimestampFilter = Union[
    TimestampCreatedTimeFilter,
    TimestampLastEditedTimeFilter,
]



from typing import Any, Literal, Union

from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr

from .base_properties._base_property import (
    NotionPropertyType,
)

from .base_properties import (
    BaseProperty,
    ButtonProperty,
    CheckboxProperty,
    CreatedByProperty,
    CreatedTimeProperty,
    DateProperty,
    EmailProperty,
    FilesProperty,
    FormulaProperty,
    LastEditedByProperty,
    LastEditedTimeProperty,
    MultiSelectProperty,
    NumberProperty,
    PeopleProperty,
    PhoneNumberProperty,
    RichTextProperty,
    SelectProperty,
    StatusProperty,
    TitleProperty,
    UniqueIdProperty,
    UrlProperty,
    VerificationProperty,
)


class Rollup(BaseModel):
    """Notionのrollup項目"""

    type: StrictStr = Field(..., description="rollupのタイプ（numberまたはarray）")
    number: StrictInt | StrictFloat | None = Field(
        None, description="数値（typeがnumberの場合のみ設定）"
    )
    date: DateProperty | None = Field(
        None, description="日付（typeがdateの場合のみ設定）"
    )
    function: StrictStr = Field(..., description="rollup関数（show_original、sum等）")
    array: (
        list[
            Union[
                ButtonProperty,
                CreatedByProperty,
                CreatedTimeProperty,
                DateProperty,
                EmailProperty,
                FilesProperty,
                NumberProperty,
                PeopleProperty,
                PhoneNumberProperty,
                RichTextProperty,
                StatusProperty,
                TitleProperty,
                UrlProperty,
                SelectProperty,
                MultiSelectProperty,
                CheckboxProperty,
                FormulaProperty,
                LastEditedByProperty,
                LastEditedTimeProperty,
                UniqueIdProperty,
                VerificationProperty,
            ]
        ]
        | None
    ) = Field(None, description="配列（typeがarrayの場合のみ設定）")

    def get_display_value(self) -> int | StrictFloat | StrictStr | bool | None:
        """
        rollup プロパティからrollupデータを取得

        Returns:
            Any: rollupデータ
        """
        match self.type:
            case "number":
                return self.number
            case "date":
                return self.date.get_display_value() if self.date else None
            case "array":
                if self.array and len(self.array) > 0:
                    return self.array[0].get_display_value()
                else:
                    return None
            case _:
                return None


class RollupProperty(BaseProperty[Literal[NotionPropertyType.ROLLUP]]):
    """Notionのrollupプロパティ"""

    type: Literal[NotionPropertyType.ROLLUP] = Field(
        NotionPropertyType.ROLLUP, description="プロパティタイプ"
    )

    rollup: Rollup = Field(..., description="rollupデータ")

    def get_display_value(self) -> str | int | float | bool | None:
        """rollupの表示値を取得

        Returns:
            StrictStr | StrictInt | StrictFloat | bool | None: rollupの表示値
        """
        return self.rollup.get_display_value()



"""
Notionカバーモデル

Database、Page、DataSourceなどで使用されるカバー画像の定義
"""

from __future__ import annotations

from enum import Enum

from pydantic import BaseModel, Field

from .file import ExternalFile, InternalFile


class CoverType(str, Enum):
    """カバータイプ"""

    EXTERNAL = "external"
    FILE = "file"


class NotionCover(BaseModel):
    """
    Notionのカバー画像（外部画像または内部ファイル）

    Database、Page、DataSourceで共通使用

    Examples:
        ```python
        # 外部画像カバー
        external_cover = NotionCover(
            type=CoverType.EXTERNAL,
            external=ExternalFile(url="https://example.com/cover.jpg")
        )

        # 内部ファイルカバー
        file_cover = NotionCover(
            type=CoverType.FILE,
            file=InternalFile(url="https://notion.so/...", expiry_time="...")
        )
        ```
    """

    type: CoverType = Field(..., description="カバータイプ")
    external: ExternalFile | None = Field(None, description="外部ファイル")
    file: InternalFile | None = Field(None, description="内部ファイル")



"""Request types for Notion API.

このモジュールは、Notion APIへのリクエストに使用する型定義を提供します。
主にPage作成・更新時のプロパティ値の設定に使用します。
"""

from .common import (
    AnnotationRequest,
    DateRequest,
    GroupObjectRequest,
    IdRequest,
    PageIconRequest,
    PageCoverRequest,
    InternalFileRequest,
    ExternalFileRequest,
    InternalOrExternalFileWithNameRequest,
    FileUploadIdRequest,
    FileUploadWithOptionalNameRequest,
    PartialUserObjectRequest,
    RelationItemRequest,
    SelectPropertyItemRequest,
)
from .page_requests import CreatePageParameters, UpdatePageParameters
from .property_requests import (
    CheckboxPropertyRequest,
    DatePropertyRequest,
    EmailPropertyRequest,
    FilesPropertyRequest,
    MultiSelectPropertyRequest,
    NumberPropertyRequest,
    PeoplePropertyRequest,
    PhoneNumberPropertyRequest,
    PropertyRequest,
    RelationPropertyRequest,
    RichTextPropertyRequest,
    SelectPropertyRequest,
    StatusPropertyRequest,
    TitlePropertyRequest,
    UrlPropertyRequest,
)

__all__ = [
    # Page requests
    "CreatePageParameters",
    "UpdatePageParameters",
    # Property requests
    "PropertyRequest",
    "TitlePropertyRequest",
    "RichTextPropertyRequest",
    "NumberPropertyRequest",
    "UrlPropertyRequest",
    "SelectPropertyRequest",
    "MultiSelectPropertyRequest",
    "PeoplePropertyRequest",
    "EmailPropertyRequest",
    "PhoneNumberPropertyRequest",
    "DatePropertyRequest",
    "CheckboxPropertyRequest",
    "RelationPropertyRequest",
    "FilesPropertyRequest",
    "StatusPropertyRequest",
    # Common types
    "IdRequest",
    "DateRequest",
    "AnnotationRequest",
    "PartialUserObjectRequest",
    "GroupObjectRequest",
    "RelationItemRequest",
    "SelectPropertyItemRequest",
    # Files and icon/cover request helpers
    "InternalFileRequest",
    "ExternalFileRequest",
    "InternalOrExternalFileWithNameRequest",
    "FileUploadIdRequest",
    "FileUploadWithOptionalNameRequest",
    "PageIconRequest",
    "PageCoverRequest",
]



from typing import Literal
from pydantic import Field

from ...models import RichTextItem
from ._base_property import BaseProperty, NotionPropertyType


class RichTextProperty(BaseProperty[Literal[NotionPropertyType.RICH_TEXT]]):
    """
    Notionのrich_textプロパティ

    複数のRichTextItemで構成される、書式付きテキストを表現するプロパティ
    """

    type: Literal[NotionPropertyType.RICH_TEXT] = Field(
        NotionPropertyType.RICH_TEXT, description="プロパティタイプ"
    )

    rich_text: list[RichTextItem] = Field(..., description="RichText要素の配列")

    def get_display_value(self) -> str | None:
        """リッチテキストの内容を取得

        Returns:
            str | None: リッチテキストの内容を連結した文字列。リッチテキストが空の場合はNone
        """
        if len(self.rich_text) == 0:
            return None
        return "".join(item.plain_text for item in self.rich_text)



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class LastEditedByPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.LAST_EDITED_BY]]
):
    """Notionのlast_edited_byプロパティ設定"""

    type: Literal[NotionPropertyType.LAST_EDITED_BY] = Field(
        NotionPropertyType.LAST_EDITED_BY, description="プロパティタイプ"
    )
    last_edited_by: EmptyObject = Field(
        default_factory=EmptyObject, description="last_edited_by設定"
    )
